#!/bin/ksh
#
# ****************************************************************************
# * Licensed Materials - Property of IBM
# * 
# * OCO Source Materials - IBM Confidential
# * 5724-C04
# *
# * Copyright IBM Corporation 2007-2011 All Rights Reserved
# *
# * The source code for this program is not published or other-
# * wise divested of its trade secrets, irrespective of what has
# * been deposited with the U.S. Copyright Office.
# *****************************************************************************

# @(#) install.sh - installs the installer (i.e. CI classes and JRE), and  
# @(#) invokes the Java CandleInstall programs.

############ Last seven mods ######################################
# @41 03/31/10 ACH Check for CandleManage running in the background 
# @40 04/08/09 JED Add PLATFORM_SUPPORTED_CHECK variable
# @39 08/24/05 TR  Feature 14459 remove remote package creation options
# @38 06/21/05 TR  Will put user in CANDLEHOME defect 4437
# @37 05/07/05 LCL Support agent deployment depot install
# @36 04/14/05 LCL Only install 32-bit GSkit; #3357
# @35 04/05/05 LCL Create /keyfiles
# @34 03/16/05 LCL Rm "Candle" from Management Server message.
# @33 03/08/05 LCL Add ITMinstall. to JREEXE invocations
# @32 03/02/05 MTL GSKit support
# @31 02/02/05 LCL QI directories get 777
# @30 01/23/05 LCL Full silent install mode.
# @29 01/22/05 MTL Disable safe-mode (obsolete for JRE 1.4.2)
# @28 12/09/04 LCL Detect/set runtime flag for Candle/IBM titles
# @27 10/28/04 LCL Rm debug writes
# @26 10/12/04 MTL Add LAP tool silent install.
# @25 09/07/04 LCL Copy terms.agr to Candlehome.
# @24 09/01/04 LCL Allow alternateJRE for LAP; use installer JRE
# @23 08/18/04 LCL LAP files moved to under /unix
# @22 07/19/04 LCL Use IBM license acceptance process (LAP).
# @21 09/05/03 LCL Export saveLang so GUI agent starts will work.
# @20 07/08/03 LCL Clarify need to stop LDAP.
# @19 04/24/03 LCL MsgEdit has its own terms.agr
# @18 04/07/03 LCL XB excluded from Roma/CCC lists cuz it's shared
# @17 12/26/02 LCL Final chmod output gets dumped.
# @16 01/03/02 LCL Roma now uses same license confirmation as CCC
# @15 12/15/01 LCL Changed Linux test for SafeMode.
# @14 12/04/01 BRC Changed to support .tdd extensions on ci fixpacks.
###################################################################

# The installer can be installed from any of the following sources:
#   1) old-style CD with external .ver files (read from the registry directory) and a "unix" product
#      directory.
#   2) new-style CD with internal .ver files (extracted from the product packages) and a "unix" product
#      directory.
#   3) Internet download with internal .ver files and no "unix" product directory.

#
# If you want to dpoebug you should uncomment the rest the line with JRE_DEBUG_ARGS not empty.
# Debug mode will suspend your VM which will be waiting for the connection with debugger.
#
JRE_DEBUG_ARGS=
#JRE_DEBUG_ARGS="-Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,server=y,address=8001,suspend=y"
#

Err_report()
{
  #set -x
  case $1 in
    #$NLS-E$ "$PROGNAME warning:  $2, continuing ..."
    warn) print -u2 "$PROGNAME `nls_replace 'KCI0535I'`:  $2, `nls_replace 'KCI0536I'` ..."
          sleep 2
          return
          ;;
	#$NLS-E$ "$PROGNAME failure:  $2."	  
    fail) print -u2 "$PROGNAME `nls_replace 'KCI0537I'`:  $2."
          cleanUp 2
          ;;
    info) [ -n "$VERBOSE" ] && print -u2 "$PROGNAME        :  $2."
          return
          ;;
    msgs) print -u2 "$PROGNAME        :  $2"
          return
          ;;
  esac
}

Use_report()
{
  print -u2 "$PROGNAME [ -h ITM home ] -v"
  #$NLS-E$ "  Displays the version and release levels of the current installer."
  print -u2 "`nls_replace 'KCI1000I'`"
  print -u2 "$PROGNAME [ -h ITM home ] [ -d PRODUCT_PATH ] [ -c ] [ -p PARAMETER_FILE ] [ -k GROUP ]"
  #$NLS-E$ "  Specify -c to print diagnostic messages to the console."
  print -u2 "`nls_replace 'KCI1001I' -c`"
  #$NLS-E$ "  Specify -d to indicate the location of the product CD."
  print -u2 "`nls_replace 'KCI1002I' -d`"
  #$NLS-E$ "  Specify -h to indicate the location of the CandleHome."
  print -u2 "`nls_replace 'KCI1003I' -h`"
  #$NLS-E$ "  Specify -j to indicate the JRE installed location. This is available on Tandem system only."
  print -u2 "`nls_replace 'KCI1004I' -j`"
  #$NLS-E$ "  Specify -p for silent mode execution using the specified PARAMETER_FILE."
  print -u2 "`nls_replace 'KCI1005I' -p PARAMETER_FILE`"
  #$NLS-E$ "  Specify -k to secure installation."
  print -u2 "`nls_replace 'KCI1340I' -k`"
  exit 2
}

locateCMD () { #--------------------------------------------------------#IZ94626
   # Find the directory where the executable ${1} resides so we can     #IZ94626
   # invoke it directly.                                                #IZ94626
   #--------------------------------------------------------------------#IZ94626
   #set -x                                                              #IZ94626
   typeset outCMD="";                                                   #IZ94626
   if [ ! -z "${1}" ]; then                                             #IZ94626
      if [ "${thisOS}" = "HP-UX" ] || [ "${thisOS}" = "SunOS" ]; then   #IZ94626
         outCMD=`which ${1} 2> /dev/null | grep -v "^no ${1}"`;         #IZ94626
      else                                                              #IZ94626
         outCMD=`which ${1} 2> /dev/null`;                              #IZ94626
      fi;                                                               #IZ94626
      if [ -z "${outCMD}" ]; then                                       #IZ94626
         outCMD=`find /bin /usr/bin /local/bin /usr/local/bin /sbin /usr/sbin /local/sbin /usr/local/sbin -name ${1} 2> /dev/null | head -n 1`; #IZ94626
      fi;                                                               #IZ94626
   fi;                                                                  #IZ94626
   print "${outCMD}";                                                   #IZ94626
}                                                                       #IZ94626
                                                                        #IZ94626 
# Run the License Acceptance Process (LAP) tool to get the user's agreement to the license

run_LAPTool()
{
  #set -x
  lapJar=lapapp.jar			# LAP jar name
  lapStatus=$CANDLEHOME/LAP/license/status.dat	# LAP status code file path
  lapCode=0
# always display license
#  [ -r $lapStatus ] && lapCode=$(grep "Status=" $lapStatus | cut -d"=" -f2)
  
  # Skip LAP if there was a previous acceptance in CandleHome
#  [ $lapCode = 9 ] && return

  # Backup any existing LAP files
  [ -d $CANDLEHOME/LAP ] && mv $CANDLEHOME/LAP $CANDLEHOME/LAP_old

  [ -z "$LAPMode" ] && LAPMode="$1 $2"

  # Prepare the LAP output area and environment
  mkdir $CANDLEHOME/LAP
  
  # Invoke LAP tool (aix requires special LIBPATH changes)
  lapJrePath=${JREEXE%%/bin*}
  [ "$(uname)" = "AIX" ] && export LIBPATH=$lapJrePath/lib:$lapJrePath/bin:$lapJrePath/bin/classic:$LIBPATH

  if [ -n "$silentMode" ]
  then
  #$NLS-E$ "\n\nUse of silent install mode implies license agreement acceptance. You can view the license file details in the $CANDLEHOME/LAP/license directory"  
   print "\n"
   nls_replace 'KCI1006I' $CANDLEHOME/LAP/license
   sleep 5
  fi

  $JREEXE -classpath $PROD_PATH/unix/LAP/LAP_HOME/$lapJar:$lapJrePath/lib/rt.jar:$lapJrePath/lib/i18n.jar $JRE_DEBUG_ARGS com.ibm.lex.lapapp.LAP $silentMode -l $PROD_PATH/unix/LAP/LA_HOME -s $CANDLEHOME/LAP $LAPMode

  # Status code=9 is the only valid acceptance code
  lapCode=$?
  [ -r $lapStatus ] && lapCode=$(grep "Status=" $lapStatus | cut -d"=" -f2)

  if [ $lapCode != 9 ]
  then
    rm -rf $CANDLEHOME/LAP
    [ -d $CANDLEHOME/LAP_old ] && mv $CANDLEHOME/LAP_old $CANDLEHOME/LAP
    if [ $lapCode = 230 ]
    then
	  #$NLS-E$ "terminating ... problem with starting Java Virtual Machine"
      Err_report fail "`nls_replace 'KCI1235E'`"         
    else	
	  #$NLS-E$ "terminating ... license declined"
      Err_report fail "`nls_replace 'KCI1007E'`"
    fi
  fi

  # Copy 3rd party license (if one exists)
  [ -f $PROD_PATH/unix/terms.agr ] && cp -p $PROD_PATH/unix/terms.agr $CANDLEHOME
  [ -f $PROD_PATH/unix/TERMS.AGR ] && cp -p $PROD_PATH/unix/TERMS.AGR $CANDLEHOME
  
  # Copy additional license files
  [ -f $PROD_PATH/license/Greek.txt ] && [ ! -f $CANDLEHOME/LAP/license/Greek.txt ] && cp -f $PROD_PATH/license/Greek.txt $CANDLEHOME/LAP/license/ > /dev/null 2>&1
  [ -f $PROD_PATH/license/Lithuanian.txt ] && [ ! -f $CANDLEHOME/LAP/license/Lithuanian.txt ] && cp -f $PROD_PATH/license/Lithuanian.txt $CANDLEHOME/LAP/license/ > /dev/null 2>&1
  [ -f $PROD_PATH/license/Russian.txt ] && [ ! -f $CANDLEHOME/LAP/license/Russian.txt ] && cp -f $PROD_PATH/license/Russian.txt $CANDLEHOME/LAP/license/ > /dev/null 2>&1
  [ -f $PROD_PATH/license/Slovenian.txt ] && [ ! -f $CANDLEHOME/LAP/license/Slovenian.txt ] && cp -f $PROD_PATH/license/Slovenian.txt $CANDLEHOME/LAP/license/ > /dev/null 2>&1
}

# Compare the levels (version and release) of two products.  The "new" product (the one to be installed)
# is the basis for the comparison with the "cur" product (the one in CANDLEHOME).

checkLevel()
{
# $1 = the name of the current .ver file.
# $2 (if specified) = the name of the new .ver file.
  #set -x
  typeset -i curRel=0 curVer=0 newRel=0 newVer=0 useVRMF=0

  curFile=$curRegPath/$1
  newFile=$newRegPath/$1
  
  [ -z "$2" ] || newFile=$newRegPath/$2

  # Ensure that both files exist.

  #$NLS-E$ "could not read \"$curFile\""
  [ -r "$curFile" ] || Err_report fail "`nls_replace 'KCI1008E' \\"$curFile\\"`"
  #$NLS-E$ "could not read \"$newFile\""
  [ -r "$newFile" ] || Err_report fail "`nls_replace 'KCI1008E' \\"$newFile\\"`"

  # Get VRMF from .ver file and remove leading whitespace and leading zero
  # Resulting VRMF will be 7 digit e.g. 6200000
  curVRMF=$(grep "^VRMF *=" $curFile | cut -d"=" -f2)
  curVer=${curVRMF##+( )+(0)}

  # If no VRMF, get Ver and Rel instead and convert to VRMF format
  if [ $curVer -eq 0 ]
  then
     curVer=$(grep "^ver *=" $curFile | cut -d"=" -f2)
     #$NLS-E$ "could not determine the version number from \"$curFile\""
     [ $curVer -ne 0 ] || Err_report fail "`nls_replace 'KCI1009E' \\"$curFile\\"`"
     curRel=$(grep "^rel *=" $curFile | cut -d"=" -f2)     
     #$NLS-E$ "could not determine the release number from \"$curFile\""
     [ -n "$curRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$curFile\\"`"
     # Convert Ver & Rel to VRMF format, this will result in 7 digit number
     # e.g. 5051000 where Ver=505 and Rel=100 i.e. VVVRRR0 
     curVer=$curVer${curRel}0
  fi
  
  # Get VRMF from new .ver file and remove leading whitespace and leading zero
  # Resulting VRMF will be 7 digit e.g. 6200000
  newVRMF=$(grep "^VRMF *=" $newFile | cut -d"=" -f2)
  newVer=${newVRMF##+( )+(0)}

  # If no VRMF, get Ver and Rel instead and convert to VRMF format
  if [ $newVer -eq 0 ]
  then
     newVer=$(grep "^ver *=" $newFile | cut -d"=" -f2)
     #$NLS-E$ "could not determine the version number from \"$newFile\""
     [ $newVer -ne 0 ] || Err_report fail "`nls_replace 'KCI1009E' \\"$newFile\\"`"
     newRel=$(grep "^rel *=" $newFile | cut -d"=" -f2)
     #$NLS-E$ "could not determine the release number from \"$newFile\""
     [ -n "$newRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$newFile\\"`"
     # Convert Ver & Rel to VRMF format, this will result in 7 digit number
     # e.g. 5051000 where Ver=505 and Rel=100 i.e. VVVRRR0 
     newVer=$newVer${newRel}0
  fi
  
  # Compare the versions (which are by now all in VRMF format)
  if [ newVer -lt curVer ]
  then
     levelStrg=Older
  elif [ newVer -gt curVer ]
  then
     levelStrg=Newer
  else
     levelStrg=Same
  fi
  
  return
}

# Compare the levels (version and release) of any two products.  The "ref" product is the basis for the 
# comparison with the "cmp" product.

checkLevel2()
{
#set -x
# $1 = the complete pathname of the reference .ver file.
# $2 = the complete pathname of the .ver file to compare with the reference.

  typeset -i refRel=0 refVer=0 cmpRel=0 cmpVer=0

  refFile=$1
  cmpFile=$2
  
  # Ensure that both files exist.

  #$NLS-E$ "could not read \"$refFile\""
  [ -r "$refFile" ] || Err_report fail "`nls_replace 'KCI1008E' \\"$refFile\\"`"
  #$NLS-E$ "could not read \"$cmpFile\""
  [ -r "$cmpFile" ] || Err_report fail "`nls_replace 'KCI1008E' \\"$cmpFile\\"`"

  # Get and compare the versions.

  refVer=$(grep "^ver *=" $refFile | cut -d"=" -f2)
  cmpVer=$(grep "^ver *=" $cmpFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the version number from \"$refFile\""
  [ -n "$refVer" ] || Err_report fail "`nls_replace 'KCI1009E' \\"$refFile\\"`"
  #$NLS-E$ "could not determine the version number from \"$cmpFile\""
  [ -n "$cmpVer" ] || Err_report fail "`nls_replace 'KCI1009E' \\"$cmpFile\\"`"

  if [ cmpVer -lt refVer ]
  then
    levelStrg2=Older
    return
  fi

  if [ cmpVer -gt refVer ]
  then
    levelStrg2=Newer
    return
  fi
 
  # If the versions are identical, compare the releases.

  refRel=$(grep "^rel *=" $refFile | cut -d"=" -f2)
  cmpRel=$(grep "^rel *=" $cmpFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the release number from \"$refFile\""
  [ -n "$refRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$refFile\\"`"
  #$NLS-E$ "could not determine the release number from \"$cmpFile\""
  [ -n "$cmpRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$cmpFile\\"`"

  # Same version, older release.
  
  if [ cmpRel -lt refRel ]
  then
    levelStrg2=Older
    return
  fi

  # Same version, same release.

  if [ cmpRel -eq refRel ]
  then
    levelStrg2=Same
    return
  fi

  # Same version, newer release.

  levelStrg2=Newer
  return
}

cleanUp()
{ # $1 should equal either the word "trap" or a number.
  #set -x

  typeset -i rc=0
  typeset name=""
  
  #$NLS-E$ "caught $1, cleanup"
  [ "$1" = "trap" ] && Err_report info "`nls_replace 'KCI1011I' \"$1\"`"
  [[ "$1" = +([0-9]) ]] && rc=$1
  
  rm -f $CANDLETMP/*.sh $CANDLETMP/*.tbl $refVFile
  
  # EJF (13046) do not remove jr*.ver files as they are used later when install is re-run
  tmpCurrDir=$(pwd)
  cd $CANDLETMP

# defect 31944, exit install we change to 4
  if [[ "$rc" != 4 ]]
  then
    rm -f $(print $(ls *.ver 2>/dev/null | grep -v '^jr')) > /dev/null 2>&1
  else
    rc=0                # reset to 0
  fi

  cd $tmpCurrDir
  
  rm -f $thisTok
  rm -f $RegBackup/Pre_CT99_Roma

  # Delete backup LAP, or restore it
  if [ -d "$CANDLEHOME/LAP" ]
  then
    [ -d "$CANDLEHOME/LAP_old" ] && rm -rf "$CANDLEHOME/LAP_old"
  else
    [ -d "$CANDLEHOME/LAP_old" ] && mv "$CANDLEHOME/LAP_old" "$CANDLEHOME/LAP"
  fi

  exit $rc  # HP-UX needs the dollar sign.
}

# Display the release level of the current! installer.

dspInstLvl()
{
  #set -x
  typeset curRel="" curVer="" regFile=""
  
  #$NLS-E$ "specify or set CANDLEHOME, and rerun"
  [ -n "$CANDLEHOME" ] || Err_report fail "`nls_replace 'KCI1012E' CANDLEHOME`"

  regFile=$CANDLEHOME/registry/$ciVFile
  #$NLS-E$ "could not read file \"$regFile\""
  [ -r $regFile ] || Err_report fail "`nls_replace 'KCI1013E' \\"$regFile\\"`"

  # Get the version and release numbers.

  curVer=$(grep "^ver *=" $regFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the version number from \"$regFile\""
  [ -n "$curVer" ] || Err_report fail "`nls_replace 'KCI1009E' \\"$regFile\\"`"
  curRel=$(grep "^rel *=" $regFile | cut -d"=" -f2)
  #$NLS-E$ "could not determine the release number from \"$regFile\""
  [ -n "$curRel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$regFile\\"`"

  #$NLS-E$ "\nThe version and release levels of the installer are:"
  echo
  nls_replace 'KCI1014I'
  print "  ver = $curVer"
  print "  rel = $curRel \n"
}

# Determine the latest (newest) fixpack for a product.

findLatest()
{ # $1 = list of fixpack .ver file names (e.g. ciux6110.ver)
  #set -x
  typeset allNames name vFile
  typeset -i lateRel=0 lateVer=0 rel=0 ver=0
  
  # Find the latest (largest) version number.
  
  for vFile in $@
  do
    ver=$(grep "^ver *=" $newRegPath/$vFile 2> /dev/null | cut -d"=" -f2)
	#$NLS-E$ "could not determine the version number from \"$newRegPath/$vFile\""
    [ -n "$ver" ] || Err_report fail "`nls_replace 'KCI1009E' \\"$newRegPath/$vFile\\"`"
    [ ver -gt lateVer ] && lateVer=ver
  done
  
  # Find which .ver files specify the latest version.
  
  for vFile in $@
  do
    name=$(grep -l "^ver *= *$lateVer" $newRegPath/$vFile 2> /dev/null)
    name=${name##*/}  # Remove the path.
    allNames="$allNames $name"
  done

  #$NLS-E$ "could not find version number \"$lateVer\" in any .ver file"
  [ -n "$allNames" ] || Err_report fail "`nls_replace 'KCI1015E' \\"$lateVer\\"`"
  
  # Find the latest (largest) release number from the latest (i.e. those with largest version number) .ver files.
  
  for vFile in $allNames
  do
    rel=$(grep "^rel *=" $newRegPath/$vFile 2> /dev/null | cut -d"=" -f2)
	#$NLS-E$ "could not determine the release number from \"$newRegPath/$vFile\""
    [ -n "$rel" ] || Err_report fail "`nls_replace 'KCI1010E' \\"$newRegPath/$vFile\\"`"
    [ rel -gt lateRel ] && lateRel=rel
  done 
  
  # Determine the name of the latest (newest) fixpack.
  
  for vFile in $allNames
  do
    lateVFile=$(grep -l "^rel *= $lateRel" $newRegPath/$vFile 2> /dev/null)
    lateVFile=${lateVFile##*/}  # Remove the path.
  done
  
  #$NLS-E$ "could not find the latest fixpack"
  [ -n "$lateVFile" ] || Err_report fail "`nls_replace 'KCI1016E'`"
}


# Copy and nls_replace to CANDLEHOME/bin and msg directory to CANDLEHOME/config/nls. 

cpMo()
{ 
    [ ! -d $CANDLEHOME/bin ] && mkdir $CANDLEHOME/bin > /dev/null 2>&1
    [ ! -d $CANDLEHOME/bin ] && Err_report fail "`nls_replace 'KCI1091E' \\"$CANDLEHOME/bin\\"`"

    [ ! -d $CANDLEHOME/config ] && mkdir $CANDLEHOME/config > /dev/null 2>&1
    [ ! -d $CANDLEHOME/config ] && Err_report fail "`nls_replace 'KCI1091E' \\"$CANDLEHOME/config\\"`"

    [ ! -d $CANDLEHOME/config/nls ] && mkdir $CANDLEHOME/config/nls > /dev/null 2>&1
    [ ! -d $CANDLEHOME/config/nls ] && Err_report fail "`nls_replace 'KCI1091E' \\"$CANDLEHOME/config/nls\\"`"
	
    cp -R $nls_replace_dir/msg $CANDLEHOME/config/nls > /dev/null 2>&1
	
    [ ! -d $CANDLEHOME/config/nls/msg ] && Err_report warn "`nls_replace 'KCI1018I' \\"msg\\" \\"$CANDLEHOME/config/nls\\"`"

    cp $nls_replace_dir/nls_replace $CANDLEHOME/bin > /dev/null 2>&1
    [ ! -a $CANDLEHOME/bin/nls_replace ] && Err_report warn "`nls_replace 'KCI1018I' \\"nls_replace\\" \\"$CANDLEHOME/bin\\"`"	
		
    chmod 755 $CANDLEHOME/bin/nls_replace > /dev/null 2>&1
	#$NLS-E$ "could not change access permissions on \"$CANDLEHOME/bin/nls_replace\""
    [ $? -eq 0 ] || Err_report warn "`nls_replace 'KCI1019I' \\"$CANDLEHOME/bin/nls_replace\\"`"
}

# Unload a single file from an archive's tmp/ directory.

unloadTmpFile()
{ # $1 = name of file to unload.
  # $2 = full pathname of archive.
  #set -x

  typeset ext tFile
  
  #$NLS-E$ "cannot read package \"$2\""
  [ -r "$2" ] || Err_report fail "`nls_replace 'KCI1020E' \\"$2\\"`"
  
  ext=${2##*.}  # Get the extension, either "tar", "ctp" or "zxx".
  
  if [[ "$ext" = t* ]] # Uncompressed tar file.
  then
    tar -$tarOpt $2 tmp/$1 > /dev/null 2>&1
    # do chown and chmod for Linux.  defect 83100
    if [[ "$chownRoot" = y ]]
    then
      for tvf in $(tar tvf $2 tmp/$1 2> /dev/null | $AWK '{print $6}')
      do
        chown root:root $tvf
        chmod 755 $tvf
      done
    fi
	#$NLS-E$ "could not unload file \"$1\""
    [ -r tmp/$1 ] || Err_report fail "`nls_replace 'KCI1021E' \\"$1\\"`"
  else  # Compressed tar file.
    zcat < $2 | tar -$tarOpt - tmp/$1 > /dev/null 2>&1
	#$NLS-E$ "could not unload file \"$1\""
    [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1021E' \\"$1\\"`"
    # do chown and chmod for Linux.  defect 83100
    if [[ "$chownRoot" = y ]]
    then
      for tvf in $(zcat < $2 | tar tvf - tmp/$1 2> /dev/null | $AWK '{print $6}')
      do
        chown root:root $tvf
        chmod 755 $tvf
      done
    fi
  fi
}

# Ensure there is enough disk space available.

checkDiskSpace()
{ # $1 contains the name of the directory to check.
  # $2 = the number of kilobytes required.
  #set -x

  typeset thisOS=$(uname -s)

  dfCmd="df -kP"  # All platforms except Solaris
  [ "$thisOS" = "SunOS" ] && dfCmd="df -k"

  $dfCmd $1 > /dev/null 2>&1  # Ensure that the file system is available.
  if [ $? -eq 0 ]
  then
    #$NLS-E$ "calculating available disk space"
    Err_report info "`nls_replace 'KCI1022I'`"

    # This was causing a very obscure problem on HP - the new method is safer and cleaner
    #set -A dfArray $($dfCmd $1 | tail -1)	# local translations cause parsing problems so lose the headers; #5663
    #kbAvail=${dfArray[3]}
    kbAvail=`$dfCmd $1 | grep "%" | head -1 | $AWK '{if (NF == 5) print $3; else if (NF == 6) print $4}'`

  else
    #$NLS-E$ "df could not determine the available disk space in \"$1\""
    Err_report fail "`nls_replace 'KCI1023E' df \\"$1\\"`"
  fi

  # This handles the case where AIX returns a match for the
  # remote filesystem name as well as the local directory.
  # It then backs off two fields from the end and grabs the space avail,
  # that is the normalized position on all platforms
  kbTmpResult=`$dfCmd /tmp 2> /dev/null | grep "%"`

  if [ "$thisOS" = "AIX" ] && [ $(print $(print "$kbTmpResult" | wc -l)) -gt 1 ]; then
    kbTmpResult=`echo "$kbTmpResult" | grep -v ':/tmp'`
  fi

  kbTmpAvail=`echo "$kbTmpResult" | $AWK '{av=NF-2; print $av}'`

  if [ -n "$kbTmpAvail" ] ; then
    #$NLS-E$ "calculating available disk space"
    Err_report info "`nls_replace 'KCI1022I'`"
  else
    #$NLS-E$ "df could not determine the available disk space in /tmp"
    Err_report fail "`nls_replace 'KCI1023E' df /tmp`"
  fi

  if [ $2 -eq 0 ]  # The value of kbAvail is still needed even if there is nothing to install.
  then
    #$NLS-E$ "\"$kbAvail\" kilobytes available"
    Err_report info "`nls_replace 'KCI1024I' \\"$kbAvail\\"`"
    return
  fi

  if [ kbAvail -gt $2 ]
  then
    #$NLS-E$ "\"$kbAvail\" kilobytes available; only \"$2\" kilobytes required for the package(s)"
    Err_report info "`nls_replace 'KCI1025I' \\"$kbAvail\\" \\"$2\\"`"
	#$NLS-E$ "OK to install"
    Err_report info "`nls_replace 'KCI1026I'`"
  else
    #$NLS-E$ "\"$2\" kilobytes required for the package(s); only \"$kbAvail\" kilobytes available"
    Err_report fail "`nls_replace 'KCI1027E' \\"$2\\" \\"$kbAvail\\"`"
  fi
}

# Determine if there are CCC and/or Roma products available for installation.

findFamilies()
{
  #set -x

  typeset pc1 pkg pTbl=proddsc.tbl tFile=tmp/proddsc.tbl
  
  typeset cccPC=$CANDLETMP/cccPC.$$ romaPC=$CANDLETMP/romaPC.$$ jarPC=$CANDLETMP/jarPC.$$ verPC=$CANDLETMP/verPC.$$
  touch $cccPC $romaPC $jarPC $verPC

  # Get all of the possible product codes from all available product tables.
  
  for pkg in $(ls $PKGLOC/ci*.tar 2> /dev/null)  # Look in the CI packages for a product description table.
  do
    tar -$tarOpt $pkg $tFile > /dev/null 2>&1
    # do chown and chmod for Linux.  defect 83100
    if [[ "$chownRoot" = y ]]
    then
      for tvf in $(tar tvf $pkg $tFile 2> /dev/null | $AWK '{print $6}')
      do
        chown root:root $tvf
        chmod 755 $tvf
      done
    fi
    [ -r $tFile ] || continue  # Forget it.
    # xb is now a product shared between CCC and Roma (LL - 4/2003)
    # CCC product codes.
    cat $CANDLEHOME/$tFile | grep -v "[Rr]oma" | egrep -v "^(ax|ci|jr|xb|\#|\*)" | cut -d"|" -f1 >> $cccPC 2> /dev/null
    # Roma product codes.
    cat $CANDLEHOME/$tFile | grep "[Rr]oma" | egrep -v "^(ax|ci|jr|xb|\#|\*)" | cut -d"|" -f1 >> $romaPC 2> /dev/null    
    break  # One is enough.
  done
  
  #  Add the codes from the product table in $CANDLEHOME/registry.
  
  if [ -r $curRegPath/$pTbl ]
  then
    # CCC product codes.
    cat $curRegPath/$pTbl | grep -v "[Rr]oma" | egrep -v "^(ax|ci|jr|xb|\#|\*)"  | cut -d"|" -f1 >> $cccPC 2> /dev/null 
    # Roma product codes.
    cat $curRegPath/$pTbl | grep "[Rr]oma" | egrep -v "^(ax|ci|jr|xb|\#|\*)" | cut -d"|" -f1 >> $romaPC 2> /dev/null    
  fi

  #$NLS-E$ "could not determine product codes"
  [[ -s "$cccPC" || -s "$romaPC" ]] || Err_report fail "`nls_replace 'KCI1028E'`"
    
  # Get the product codes from the available jar files.
  
  ls $PKGLOC/*.j* 2> /dev/null | sed "s?.*/??" | cut -c1-2 > $jarPC
  [ -s $jarPC ] || find $PKGLOC -name "*.j*" 2> /dev/null | sed "s?.*/??" | cut -c1-2 > $jarPC
  #$NLS-E$ "could not find any jar files in \"$PKGLOC\""
  [ -s $jarPC ] || Err_report fail "`nls_replace 'KCI1029E' \\"$PKGLOC\\"`"
  sort -u $jarPC -o $jarPC 2> /dev/null  # Keep only unique product codes.
  
  # Get the installed product codes from the Registry
  
  ls $curRegPath/*.ver 2> /dev/null | sed "s?.*/??" | cut -c1-2 > $verPC
  [ -s $verPC ] && sort -u $verPC -o $verPC 2> /dev/null  # Keep only unique product codes.
  
  # See if there are CCC products available.
  
  if [[ -r "$cccPC" && -s "$cccPC" ]]
  then
    sort -u $cccPC -o $cccPC 2> /dev/null  # Keep only unique product codes.
    
    # Search the install media
    for pc1 in $(< $cccPC)
    do
      foundSth=$(grep $pc1 $jarPC 2> /dev/null)
      if [ -n "$foundSth" ]
      then
        hasCCC=true  # hasCCC is declared in Main.
        break
      fi
    done

    # Search the installed products (if any) 
    if [ -s $verPC ]
    then
      for pc1 in $(< $cccPC)
      do
        foundSth=$(grep $pc1 $verPC 2> /dev/null)
        if [ -n "$foundSth" ]
        then
          hadCCC=true  # hadCCC is declared in Main.
          break
        fi
      done
    fi
  fi

  # See if there are Roma products available.
  
  if [[ -r "$romaPC" && -s "$romaPC" ]]
  then
    sort -u $romaPC -o $romaPC 2> /dev/null  # Keep only unique product codes.
    
    # See if this is a RQ bundling of Roma (Candle Application Services Pac)
    foundSth=$(grep "rq" $jarPC 2> /dev/null)
    [ -n "$foundSth" ] && hasCASP=true  # hasCASP is declared in Main.

    # See if this package has an LDAP (for later eBP warning)
    foundSth=$(grep "xs" $jarPC 2> /dev/null)
    [ -n "$foundSth" ] && hasLDAP=true  # hasLDAP is declared in Main.

    # Search the install media
    for pc1 in $(< $romaPC)
    do
      foundSth=$(grep $pc1 $jarPC 2> /dev/null)
      if [ -n "$foundSth" ]
      then
        hasRoma=true  # hasRoma is declared in Main.
        break
      fi
    done

    # Search the installed products (if any) 
    if [ -s $verPC ]
    then
      # See if this is a RQ bundling of Roma (Candle Application Services Pac)
      foundSth=$(grep "rq" $verPC 2> /dev/null)
      [ -n "$foundSth" ] && hadCASP=true  # hadCASP is declared in Main.

      for pc1 in $(< $romaPC)
      do
        foundSth=$(grep $pc1 $verPC 2> /dev/null)
        if [ -n "$foundSth" ]
        then
          hadRoma=true  # hadRoma is declared in Main.
          break
        fi
      done
    fi  
  fi  

  #if package is not roma by default set CCC.
  #New agents can have no entry in proddsc.tbl. New entry will be added during install process.
  [ -n "$hasRoma" ] || hasCCC=true


  # See if CMS is in the packaging
  foundSth=$(grep "ms" $jarPC 2> /dev/null)
  [ -n "$foundSth" ] && hasCMS=true  # hasCMS is declared in Main.

  rm -f $cccPC $jarPC $romaPC $verPC
}  

# Unload a .tar  or .ctpfile.
# @14

unTarPkg()
{ # $1 = full pathname of the file to untar.

  typeset -i i=0 maxTry=2
  typeset ext rc

  #$NLS-E$ "cannot read package \"$1\""
  [ -r "$1" ] || Err_report fail "`nls_replace 'KCI1030E' \\"$1\\"`"

  ext=${1##*.}  # ext (extension) will be either "ctp", "tar", tdd or "zdd" where d = a digit.
  while [ i -lt maxTry ]
  do
    if [[ "$ext" = t* ]]  # Uncompressed tar file.
    then
      tar -$tarOpt $1 > /dev/null 2>&1
      rc=$? 
      # do chown and chmod for Linux.  defect 83100
      if [[ "$chownRoot" = y ]]
      then
        for tvf in $(tar tvf $1 2> /dev/null | $AWK '{print $6}')
        do
          chown root:root $tvf
          chmod 755 $tvf
        done
      fi
      [ -r $1 ] || rc=-1
    else  # Compressed tar file.
      zcat < $1 | tar -$tarOpt - > /dev/null 2>&1
      rc=$?
      # do chown and chmod for Linux.  defect 83100
      if [[ "$chownRoot" = y ]]
      then
        for tvf in $(zcat < $1 | tar tvf - 2> /dev/null | $AWK '{print $6}')
        do
          chown root:root $tvf
          chmod 755 $tvf
        done
      fi
    fi

    if [ "$rc" = "0" ]
    then
      return
    else
      i=i+1
    fi
  done

  # If we get to here, the untar may have failed.

  #$NLS-E$ "unarchive of \"$1\" may have failed"
  Err_report warn "`nls_replace 'KCI1031I' \\"$1\\"`"
}

# Copy and rename the readme file.

cpRMFile()
{
:
#  typeset docDir=$CANDLEHOME/doc rmName=readme.txt
#  typeset desc rel toName
#  
#  if [ -f $PKGLOC/$rmName ]
#  then
#
#    desc=$(grep "Readme" $PKGLOC/$rmName 2> /dev/null | head -1)
#    if [ -z "$desc" ]
#    then
#      desc=$(grep "README" $PKGLOC/$rmName 2> /dev/null | head -1)
#      if [ -z "$desc" ]
#      then
#        Err_report warn "cannot copy \"$rmName\" file"
#        return
#      fi
#    fi
#
#    # Determine the product name.
#    
#    desc=${desc#*for}  # Remove the boilerplate.
#    desc=${desc%-*}    # Keep just the product name.
#    desc=$(print $desc | sed "s/  */_/g")  # Replace consecutive spaces with a single underscore.
#    desc=$(print $desc | sed "s,/,--,g")  # Replace slash with a two dashes. @12
#    
#    # Determine the release number.
#    
#    rel=$(grep "Release Information:" $PKGLOC/$rmName 2> /dev/null | cut -d":" -f2 | sed "s/  *//g")
#    if [ -z "$rel" ]
#    then
#      Err_report warn "cannot copy \"$rmName\" file"
#      return
#    fi
#    
#    # Copy the readme to the CANDLEHOME/doc directory.
#    
#    cp $PKGLOC/$rmName $docDir/readme_${desc}_$rel.txt > /dev/null 2>&1
#    [ $? -eq 0 ] || Err_report warn "could not copy the \"$rmName\" file"
#  fi
}

# Ask the user what readme file they want to view

execRMDialog()
{
  #set -x
  typeset -i i=1
  
  # Build a list of descriptions.
  
  for rmf in $(ls $CANDLEHOME/doc/readme*.txt)
  do
    desc=${rmf##*/}       # Remove the pathname.
    desc=${desc#readme_}  # Remove the boilerplate.
    desc=${desc%.txt}     # Ditto for the extension.
    desc=$(print $desc 2> /dev/null | sed "s/_/ /g")  # Replace the underscores with blanks.
    desc=$(print $desc 2> /dev/null | sed "s,--,/,g")  # Replace two dashes with slash. @12
    rmDescs[i]="$desc"
    i=i+1
  done
  
  #$NLS-E$ "Exit this menu."
  rmDescs[i]="`nls_replace 'KCI1032I'`"  # Add an escape clause.
  maxVal=$i
  
  # Generate a "select" menu.
  
  i=1
  #$NLS-E$ "\nSelect one of the following readme files: \n"
  echo
  nls_replace 'KCI1033I'
  echo
  while [ i -le maxVal ]
  do
    print "$i) ${rmDescs[i]}"
    i=i+1
  done
  
  # Force the user to make a selection.
  
  while [ anything ]
  do
    #$NLS-E$ "Please enter a valid number:  \c"
    print "`nls_replace 'KCI1034I'` \c"
    read rmIdx
    [[ $rmIdx = +([0-9]) ]] || continue  # Make sure it's a number.
    [[ "$rmIdx" -gt 0 && "$rmIdx" -le maxVal ]] && break  # It's a valid number.
  done
}

# Version all CI elements with respect to the base package.

checkCIElements()
{ # $1 = names of all elements without an extension (e.g. cienv1).

  #set -x
  typeset ele="" ext=tar
  
  for ele in $@
  do
    #$NLS-E$ "cannot read element package \"$PKGLOC/$ele.$ext\""
    [ -r "$PKGLOC/$ele.$ext" ] || Err_report fail "`nls_replace 'KCI1035E' \\"$PKGLOC/$ele.$ext\\"`"

    if [ "$hasRegDir" = "no" ]  # If necessary, extract the .ver file.
    then
      unloadTmpFile $ele.ver $PKGLOC/$ele.$ext 
    fi
    
	#$NLS-E$ "cannot read file \"$newRegPath/$ele.ver\""
    [ -r "$newRegPath/$ele.ver" ] || Err_report fail "`nls_replace 'KCI1036E' \\"$newRegPath/$ele.ver\\"`"
    
    # Compare the level of the element with that of the base package.  Install only if they are the same.
    
    checkLevel2 $newRegPath/$ciVFile $newRegPath/$ele.ver
	#$NLS-E$ "element package \"$ele\" at different level than base package"
    [ "$levelStrg2" = "Same" ] || Err_report fail "`nls_replace 'KCI1037E' \\"$ele\\"`"
  done
}

# Version all CI fixpack elements with respect to the base fixpack.

checkCIFixElements()
{ # $1 = name of base fixpack without an extension (e.g. ciux6110)
  # $2 = names of all element packages with extensions (e.g. ciux6110.t01).

  #set -x
  typeset baseName=$1 ele="" ext=""
  
  shift  # Skip the base.
  for ele in $@
  do
    ext=${ele##*.}  # Get the extension.
    ele=${ele%.*}$ext  # Construct the .ver file name (e.g. ciux6110t01).
	#$NLS-E$ "cannot read file \"$newRegPath/$ele.ver\""
    [ -r "$newRegPath/$ele.ver" ] || Err_report fail "`nls_replace 'KCI1036E' \\"$newRegPath/$ele.ver\\"`"
    
    # Compare the level of the element with that of the base package.  Install only if they are the same.
    
    checkLevel2 $newRegPath/$baseName.ver $newRegPath/$ele.ver
	#$NLS-E$ "element package \"$ele\" at different level than base package"
    [ "$levelStrg2" = "Same" ] || Err_report fail "`nls_replace 'KCI1037E' \\"$ele\\"`"
  done
}

# Version all JR elements with respect to the base package.

checkJRElements()
{ # $1 = names of all elements with an extension (e.g. jraix4.z01).

  #set -x
  typeset bName="" ele="" ext="" verName=""
  
  for ele in $@
  do
    #$NLS-E$ "cannot read element package \"$PKGLOC/$ele\""
    [ -r "$PKGLOC/$ele" ] || Err_report fail "`nls_replace 'KCI1038E' \\"$PKGLOC/$ele\\"`"

    # Construct the name of the .ver file.
    
    ext=${ele##*.}          # ext -> z01
    bName=${ele%.*}         # bName -> jraix4
    verName=$bName$ext.ver  # verName -> jraix4z01.ver
    
	#$NLS-E$ "cannot read file \"$newRegPath/$verName\""
    [ -r "$newRegPath/$verName" ] || Err_report fail "`nls_replace 'KCI1036E' \\"$newRegPath/$verName\\"`"
    
    # Compare the level of the element with that of the base package.  Install only if they are the same.
    
    checkLevel2 $newRegPath/$bName.ver $newRegPath/$verName
	#$NLS-E$ "element package \"$ele\" at different level than base package"
    [ "$levelStrg2" = "Same" ] || Err_report fail "`nls_replace 'KCI1037E' \\"$ele\\"`"
  done
}

# See if there are any fixpacks in the product directory.

lookForFixPacks()
{
  # For now (6/22/2005) make it appear there are no fixpacks because fixpack design isn't complete
  # The test below will think some std packages are fixpacks based on the test for an 8 char name
  # of which the last 4 char must be digits.  However, new platform codes will cause this to 
  # fail, such as axli6243.jar ... the 6243 fits the fixpack test, but is actually the platform code
  # for intel linux (kernel 2.4, 32-bit)
 
  #set -x 
  hasFixes=false
  return

  # Define local variables.
  
  typeset l4 name pkg
  typeset -i len=0
  
  ls $PKGLOC/*.j* > /dev/null 2>&1  # Redundant since this check has already been made.
  [ $? -eq 0 ] || return
  
  # Determine if there are any fixpacks available.
  
  for name in $(ls $PKGLOC/*.j* 2> /dev/null)
  do
    pkg=${name##*/}  # Remove the path.
    pkg=${pkg%.*}    # Remove the .jar extension.
    len=${#pkg}
    [ len -eq 8 ] || continue  # Fixpack names sans extension are always 8 characters.
    l4=$(print $pkg | cut -c5- 2> /dev/null)
    if [[ $l4 = @([0-9][0-9][0-9][0-9]) ]]  # The last 4 characters in a fixpack name are digits.
    then
      hasFixes=true
      return
    fi
  done
}  
  
# Install products or maintenance from the GUI.

startGUI()
{
  #$NLS-E$ "Please set DISPLAY environmental variable, and rerun $PROGNAME"
  [ -n "$DISPLAY" ] || Err_report fail "`nls_replace 'KCI1039E' DISPLAY $PROGNAME`"
    
  cd $CANDLEHOME/bin > /dev/null 2>&1
  #$NLS-E$ "could not cd to \"$CANDLEHOME/bin\" directory"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI0683E' \\"$CANDLEHOME/bin\\"`"
   
  export PATH=$savePath:/usr/bin/X11:/usr/openwin/bin

  $JREEXE -classpath $CLASSPATH $JRE_DEBUG_ARGS $SAFE_MODE $DEPLOY_MODE ${SUSE_VERSION} -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.JFrame1 -dsa $kbAvail -ca $cArch -ch $CANDLEHOME -cd $PKGLOC -ad "$archDesc" $DEBUG_CON $instFam $tivoli

  [ $? -eq 0 ] || cleanUp 2
}

# Install support from command Line

startSupport()
{
  typeset -i rc=0

  # Create the reference .ver file.  This file is used to determine if anything was installed.

  touch $refVFile > /dev/null 2>&1
  #$NLS-E$ "could not create reference version file"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1040E'`"

  cd $CANDLEHOME/bin > /dev/null 2>&1
  #$NLS-E$ "could not cd to \"$CANDLEHOME/bin\" directory"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI0683E' \\"$CANDLEHOME/bin\\"`"

  # Silent mode passes parameter filename to Java
  silentParmFile=""
  [ -n "$silentMode" ] && silentParmFile="-p $PARM_FILE"

  $JREEXE -classpath $CLASSPATH $JRE_DEBUG_ARGS $SAFE_MODE $DEPLOY_MODE ${SUSE_VERSION} -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.CandleInstall -ch $CANDLEHOME -dl $PKGLOC -sa $kbAvail -ty supp -ca $cArch -ta $kbTmpAvail $DEBUG_CON $instFam $tivoli  $silentParmFile -i $(print $CDNAME | cut -c1-6) -k $AUTHKEY # Run the command.
  rc=$?
}
# Install products or maintenance from the command line.

startCmdLn()
{ # $1 will be either "fix" or "prod"
  #set -x
  
  typeset -i rc=0

  # Create the reference .ver file.  This file is used to determine if anything was installed.

  touch $refVFile > /dev/null 2>&1
  #$NLS-E$ "could not create reference version file"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1040E'`"

  cd $CANDLEHOME/bin > /dev/null 2>&1
  #$NLS-E$ "could not cd to \"$CANDLEHOME/bin\" directory"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI0683E' \\"$CANDLEHOME/bin\\"`"

  # Silent mode passes parameter filename to Java
  silentParmFile=""
  [ -n "$silentMode" ] && silentParmFile="-p $PARM_FILE"

  $JREEXE -classpath $CLASSPATH $JRE_DEBUG_ARGS $SAFE_MODE $DEPLOY_MODE ${SUSE_VERSION} -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.CandleInstall -ch $CANDLEHOME -dl $PKGLOC -sa $kbAvail -ty $1 -ca $cArch -ta $kbTmpAvail $DEBUG_CON $instFam $tivoli  $silentParmFile -i $(print $CDNAME | cut -c1-6) -k $AUTHKEY # Run the command.

  rc=$?

  # Call to cleanup some files we have around
  cleanupSystem

  if [ rc -eq 2 ]  # Roma specific processing.
  then
    $CANDLEHOME/bin/cannoxterm.sh romaoa $CANDLEHOME
  else
    [ rc -eq 0 ] || cleanUp 2
  fi

  [ "$instFam" = "-ccc" ] || return
  # Set permissions on certain agents.

  # No SetPerm required for Tandem system
  if [[ "$(uname -s)" != "NONSTOP_KERNEL" ]]
  then
  cmd=$CANDLEHOME/bin/SetPerm
  if [ ! -x $cmd ]
  then
    #$NLS-E$ "command \"$cmd\" not found"
    Err_report warn "`nls_replace 'KCI1041I' \\"$cmd\\"`"

  else
    # run SetPerm only if root user
    if [ "${thisRealUser}" = "root" ]; then                             #IZ94626.1
      saveMask="$(umask)"
      umask 0022        # APAR 89533 - Don't create links directory with 777 permission.
      $cmd -a  # Run the command (-a to suppress output)
      umask $saveMask   # APAR 89533 - Don't affect other things
    else 
      # For non-root user, just display a warning
      echo
	  #$NLS-E$ "The root user must run bin/SetPerm from the ITM home to complete the setup"
      Err_report warn "`nls_replace 'KCI1042I' bin/SetPerm`"
    fi
  fi	#end ! -x $cmd
  fi	#end uname -s

  # Only print the CandleConfig message if something was installed.

  for file in $(ls $curRegPath/*.ver)
  do
    if [ "$file" -nt "$refVFile" ]  # Products have been installed.
    then
      if [[ $file != *@(tpa|tms|tpw|tpd|tps).ver ]]	      
      then  
        #$NLS-E$ "\n You may now configure any locally installed IBM Tivoli Monitoring product via"
        #$NLS-E$ " the \"$CANDLEHOME/bin/itmcmd config\" command. \n"
	    echo
	    nls_replace 'KCI1045I' \""$CANDLEHOME/bin/itmcmd config"\"
	    echo
	    break
      fi
    fi
  done
}

# Create remote packages via GUI.

startGUIRP()
{
  #$NLS-E$ "Please set DISPLAY environmental variable, and rerun $PROGNAME"
  [ -n "$DISPLAY" ] || Err_report fail "`nls_replace 'KCI1046E' DISPLAY $PROGNAME`"
  
  export PATH=$savePath:/usr/bin/X11:/usr/openwin/bin
 
#  print $JREEXE -classpath $SAFE_MODE $CLASSPATH ITMinstall.jclone_agents -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -ad "$archDesc" -pl $PKGLOC -td $tarDir -pm $prodMode $DEBUG_CON
  
  $JREEXE -classpath $JRE_DEBUG_ARGS $SAFE_MODE $CLASSPATH -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.jclone_agents -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -ad "$archDesc" -pl $PKGLOC -td $tarDir -pm $prodMode $DEBUG_CON
  [ $? -eq 0 ] || cleanUp 2
  
  cd $CANDLEHOME/bin
  restart_proc start
  cd - 2> /dev/null > /dev/null
  
  cleanUp 0
}

# Create remote packages via the command line.

startCmdLnRP()
{
  #set -x 
#  print $JREEXE -classpath $CLASSPATH $SAFE_MODE ITMinstall.CandleClone -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -pl $PKGLOC -td $tarDir -pm $prodMode $DEBUG_CON
  
  $JREEXE -classpath $CLASSPATH $JRE_DEBUG_ARGS $SAFE_MODE -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall.CandleClone -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -pl $PKGLOC -td $tarDir -pm $prodMode $DEBUG_CON
  [ $? -eq 0 ] || cleanUp 2

  cd $CANDLEHOME/bin
  restart_proc start
  cd - 2> /dev/null > /dev/null
  
  cleanUp 0
}

# Install packages to the agent depot via the command line.

startCmdLnDepot()
{
  PROPS=-Ddebugcon=true 
  if [ "$DEBUG_CON" = "" ]
  then
     PROPS=-Ddebugcon=false
  fi
  $JREEXE $PROPS -classpath $CLASSPATH $JRE_DEBUG_ARGS -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall/AgentBundleDepotInstall -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -pl $PKGLOC $DEBUG_CON
  [ $? -eq 0 ] || cleanUp 2

  cd $CANDLEHOME/bin
  restart_proc start
  cd - 2> /dev/null > /dev/null
  
  cleanUp 0
}

# Install packages to the agent depot via the GUI.

startGUIDepot()
{ 
  PROPS=-Ddebugcon=true 
  #set-x
  if [ "$DEBUG_CON" = "" ]
  then
     PROPS=-Ddebugcon=false
  fi
  $JREEXE $PROPS -classpath $CLASSPATH $JRE_DEBUG_ARGS -DGlobalDbgLevel="$TRACE_LEVEL" -DInstallRASConfig="$CANDLEHOME/config/ITMInstallRAS.properties" ITMinstall/AgentBundleDepotInstall -ch $CANDLEHOME -dsa $kbAvail -ca $cArch -pl $PKGLOC -m gui $DEBUG_CON
  [ $? -eq 0 ] || cleanUp 2

  cd $CANDLEHOME/bin
  restart_proc start
  cd - 2> /dev/null > /dev/null
  
  cleanUp 0
}

# Delete old token files, and warn if there is another install running in this <candlehome>.

procTokFiles()
{
  #set -x
  typeset -i maxAge=5  # Remove "remote" token files older than 5 days.
  typeset -i age=0 dot=0 doy=0
  typeset PID name
  
  foundSth=$(ls ${baseTok}* | grep -v $thisTok 2> /dev/null) # Other installs?
  [ -n "$foundSth" ] || return
    
  doy=$(date +"%j")  # Day of the year (001 - 366)
  
  # Remove token files:
  # 1) created on this machine that are no longer connected to any process.
  # 2) created on other machines that are older than maxAge.
  
  for name in $(ls ${baseTok}* | grep -v $thisTok)
  do
    foundSth=$(cat $name | grep "$thisMach" 2> /dev/null)
    if [ -n "$foundSth" ]  # The token file was created on this machine.
    then
      PID=${name##*_}
      foundSth=$(ps -p $PID | grep $PID 2> /dev/null)    
      [ -z "$foundSth" ] && rm -f $name
    else  
      foundSth=$(cat $name | grep "|" 2> /dev/null) # Ensure the day of year field exists.
      [ -n "$foundSth" ] || continue
      
      dot=$(cat $name | cut -d"|" -f2 | cut -d"|" -f1)  # Day the token file was created (001 - 366).

      if [ doy -ge dot ]
      then
        age=doy-dot
      else  # This could happen towards the end of a year, so "normalize" the day of the year.
        doy=366+doy
        age=doy-dot
      fi

      [ age -gt maxAge ] && rm -f $name
    fi
  done
  
  # Warn the user about possible simultaneous installs.
  
  foundSth=$(ls ${baseTok}* | grep -v $thisTok 2> /dev/null) # Check for other installs.
  if [ -n "$foundSth" ] 
  then
    #$NLS-E$ "\nAn install may already be running in \"$CANDLEHOME\" from"
    #$NLS-E$ "the following machine(s): \n"
    echo
	nls_replace 'KCI1047I' \"$CANDLEHOME\"
	echo
	for name in $(ls ${baseTok}* | grep -v $thisTok)
    do
      print "  \c"; cat $name
    done
  
    repeat=1
    while [ $repeat -eq 1 ]; do
      #$NLS-E$ "\nContinue with this installation [ y or n; \"n\" is default ]?  \c"
      echo
	  print "`nls_replace 'KCI1048I'` [1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"2\" `nls_replace 'KCI0199I'` ]?  \c"
      read
      [ -z "$REPLY" ] && REPLY=1
      case $REPLY in
        y*|Y*|1) repeat=0 
               ;;
        n*|N*|2) repeat=0
               rm -f $thisTok
               cleanUp 0
               ;;       
      esac
    done
  fi
}

######################################################
# routine to sort lines representing running processes
# before they are stopped or started
# Parameters:
# 1. Mode, start or stop string
# 2. OriginalLines, list lines in the format: code$fieldSep... or code ...
# Output:
# orderedProcesses, array that contains all lines from 'OriginalLines' but ordered in desired way
order_proc()
{
	#set -x
	Mode=$1 
	shift
	OriginalLines=$*
	cqLine=""
 	kfLine="" 
	osLine="" 
	otherLines="" 
	msLine=""
	for proc in $OriginalLines
	do
		[ -n "$fieldSep" ] && Code=$(print "$proc" | cut -d "$fieldSep" -f1) || Code=$(print $proc | cut -c1-2)
		
		case "$Code" in
		cq) cqLine=$proc ;;
		kf) kfLine=$proc ;;
		ux|lz) osLine="$osLine $proc" ;;
		ms) msLine=$proc ;;
		*) otherLines="$otherLines $proc";;
		esac	
	done

	cienvVRMF=$(grep "^VRMF *=" $CANDLEHOME/registry/cienv.ver 2> /dev/null | cut -d"=" -f2)
	cienvVer=${cienvVRMF##+( )+(0)}

	if [[ -z $cienvVer ]]
	then
		cienvVer=$(grep "^ver *=" $CANDLEHOME/registry/cienv.ver 2> /dev/null | cut -d"=" -f2)
		cienvVer=${cienvVer##+( )}

		cienvRel=$(grep "^rel *=" $CANDLEHOME/registry/cienv.ver 2> /dev/null | cut -d"=" -f2)
		cienvRel=${cienvRel##+( )}

		cienvVer=$cienvVer${cienvRel}0
	fi

	archPrefix=${cArch%%+([0-9])}
	tepsVerFile=$CANDLEHOME/registry/cq${archPrefix}*.ver

	tepsVRMF=$(grep "^VRMF *=" $tepsVerFile 2> /dev/null | cut -d"=" -f2)
	tepsVer=${tepsVRMF##+( )+(0)}

	if [[ -z $tepsVer ]]
	then
		tepsVer=$(grep "^ver *=" $tepsVerFile 2> /dev/null | cut -d"=" -f2)
		tepsVer=${tepsVer##+( )}

		tepsRel=$(grep "^rel *=" $tepsVerFile 2> /dev/null | cut -d"=" -f2)
		tepsRel=${tepsRel##+( )}

		tepsVer=$tepsVer${tepsRel}0
	fi

	if [[ $tepsVer -ge "6200000" && $cienvVer -ge "6230000" ]]
	then
		if [ "$Mode" = "stop" ]
		then
			orderedProcesses="$cqLine $osLine $otherLines $msLine"
		else
			orderedProcesses="$msLine $otherLines $osLine $cqLine"
		fi
	else
		if [ "$Mode" = "stop" ]
		then
			orderedProcesses="$cqLine $kfLine $osLine $otherLines $msLine"
		else
			orderedProcesses="$msLine $otherLines $osLine $kfLine  $cqLine"
		fi
	fi
}

# routine to stop or start process.
restart_proc()
{
	#set -x
	Code=""
	Instance=""
	Userid=""
	OSAgentUserid=""
	ret=0
	typeset RestartOSAgent="NO"
	if [ -n "$DEPLOY_MODE" ]
	then
		trap "true" 13 15  # Set trap.
		trap "cleanUp trap" 1 2 3  # Set trap.
	else
		trap "cleanUp trap" 1 2 3 15  # Set trap.
	fi

	# Gather agent codes and sort it in the order: TEPS, EHS, Os Agent, other agents, TEMS
	order_proc $1 $fld23
	for proc in $orderedProcesses
	do
		[ -n "$fieldSep" ] && Code=$(print "$proc" | cut -d "$fieldSep" -f1) || Code=$(print $proc | cut -c1-2)
		[ -n "$fieldSep" ] && PID=$(print "$proc" | cut -d "$fieldSep" -f2) || PID=$(print $proc | cut -c3-)
		[ -n "$fieldSep" ] && Instance=$(print "$proc" | cut -d "$fieldSep" -f3) || Instance=$(print $proc | cut -c4-)

		# IZ17921 need to check userid on running processes
		[ -n "$fieldSep" ] && Userid=$(print "$proc" | cut -d "$fieldSep" -f4) || Userid=$(print $proc | cut -c5-)

		# Get the product description from the proddsc.tbl file.  If there are running processes,
		# then this file should exist.
    
		if [[ -r $pFile ]]  
		then
			desc=$(grep "^$Code|" $pFile |head -1 | cut -d"|" -f2)  # desc -> "NetView/AIX Alert Adapter"
			[ -n "$desc" ] || desc=$Code  # Just in case.
		fi
	
		# Don't restart TEMS, TEPS and TEP if deploy
		if [[ "$Code" = ms ]]
		then
			if [[ -z "$DEPLOY_MODE" ]] 
			then 
				# IZ17921 add userid if different
				if [[ "${thisRealUser}" = "${Userid}" ]]; then  #IZ94626.1
					./CandleServer $1 $Instance
				else
					if [[ -n "${su_value}" ]]; then           #IZ94626.1
						$su_value - $Userid -c "$CANDLEHOME/bin/CandleServer $1 $Instance"
					else
						#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
						Err_report warn "`nls_replace 'KCI0246I' 'su'`"    #IZ94626.1
					fi
				fi
				CandleServerRet=$?
				#defect 54036
				if [[ $CandleServerRet != 0 ]] 
				then
					# when TEMS wasn't stopped/started then we return -1 - this will used only to stop installation in that case. 
					ret=1
				fi
			fi
		else
		    if [[ -n "$DEPLOY_MODE" ]]
		    then
		      [[ "$Code" = @(cq|cj) ]] && continue
		    fi
			
			# check if option -f is supported 
			caOpt=""
			if [[ "$1" = "stop" ]]
			then
				currentFile=$CANDLEHOME/registry/cienv.ver
				currentVer=$(grep "^ver *=" $currentFile 2> /dev/null | cut -d"=" -f2)
				if [[ "$currentVer" -ge "620" ]];
				then
					[[ "$Code" = "kf" || -n "$DEPLOY_MODE" ]] && caOpt="-f"
				fi  						
			fi
			
			if [[ $Instance != "None" ]]
			then
				if [[ $Code = "um" ]]
				then
                          # IZ17921 add userid if different
                          if [[ "${thisRealUser}" = "${Userid}" ]]; then #IZ94626.1
				    ./CandleAgent $caOpt -p $Instance $1 $Code
                          else
					if [[ -n "${su_value}" ]]; then           #IZ94626.1
						$su_value - $Userid -c "$CANDLEHOME/bin/CandleAgent $caOpt -p $Instance $1 $Code"
					else
						#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
						Err_report warn "`nls_replace 'KCI0246I' 'su'`"    #IZ94626.1
					fi
                          fi
				else
					if [[ $Code = "ud" && $1 = "stop" ]]
					then
						hostNameInRunInfo=$(print "$fld123" | grep '|'$PID'|' | cut -d "$fieldSep" -f1 | head -1)
					fi 
					Instance=${Instance#${hostNameInRunInfo}_}
					# IZ17921 add userid if different
					if [[ "${thisRealUser}" = "${Userid}" ]]; then #IZ94626.1
						./CandleAgent $caOpt -o $Instance $1 $Code
					else
						if [[ -n "${su_value}" ]]; then     #IZ94626.1
							$su_value - $Userid -c "$CANDLEHOME/bin/CandleAgent $caOpt -o $Instance $1 $Code"
						else
							#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
							Err_report warn "`nls_replace 'KCI0246I' 'su'`"    #IZ94626.1
						fi
					fi
				fi	
			else
				if [[ "$1" = "start" ]]
				then
					if [[ $Code = "lz" ]]
					then
	                  		RestartOSAgent=$Code
						OSAgentUserid=$Userid
					else
						if [[ $Code = "ux" ]]
						then
      	            			RestartOSAgent=$Code
							OSAgentUserid=$Userid
						else
						    startAgent=yes
						    [[ $Code = "sy" ]] && [[ -n "$syNotStart" ]] && startAgent=no
						    [[ $Code = "hd" ]] && [[ -n "$hdNotStart" ]] && startAgent=no
						    if [[ "$startAgent" = "yes" ]]
						    then										
							# IZ17921 add userid if different
							if [[ "${thisRealUser}" = "${Userid}" ]]; then #IZ94626.1
								./CandleAgent $1 $Code
							else
								if [[ -n "${su_value}" ]]; then #IZ94626.1
									$su_value - $Userid -c "$CANDLEHOME/bin/CandleAgent $1 $Code"
								else
									#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
									Err_report warn "`nls_replace 'KCI0246I' 'su'`" #IZ94626.1
								fi
							fi
						    fi
						fi
					fi
				else										
					# IZ17921 add userid if different
					if [[ "${thisRealUser}" = "${Userid}" ]]; then #IZ94626.1
						./CandleAgent $caOpt $1 $Code
					else
						if [[ -n "${su_value}" ]]; then     #IZ94626.1
							$su_value - $Userid -c "$CANDLEHOME/bin/CandleAgent $caOpt $1 $Code"
						else
							#$NLS-E$ "Unable to execute su as non-root userid" #IZ94626.1
							Err_report warn "`nls_replace 'KCI0246I' 'su'`"    #IZ94626.1
						fi
					fi
				fi
			fi
	      fi
	done

	if [[ $RestartOSAgent != "NO" ]]; then                            #IZ94626.1
         restartcmd="${CANDLEHOME}/bin/CandleAgent ${1} ${Code}";       #IZ94626.1
         if [[ "${thisRealUser}" = "${Userid}" ]]; then                 #IZ94626.1
            ${restartcmd};                                              #IZ94626.1
         else                                                           #IZ94626.1
	      if [[ -n "${su_value}" ]]; then                             #IZ94626.1
	         $su_value - ${Userid} -c "${restartcmd}";                #IZ94626.1
	      else                                                        #IZ94626.1
	         #$NLS-E$ "Unable to execute su as non-root userid"       #IZ94626.1
	         Err_report warn "`nls_replace 'KCI0246I' 'su'`"          #IZ94626.1
	      fi                                                          #IZ94626.1
         fi                                                             #IZ94626.1
	fi                                                                #IZ94626.1
	return $ret
}

cinfoRunning() { #------------------------------------------------------#IZ96646
   # Obtain and convert "cinfo -r" output into the following form:      #IZ96646
   #                                                                    #IZ96646
   #     hostname|pc|pid|instance|realuser                              #IZ96646
   #                                                                    #IZ96646
   # On Solaris, "ps -f" returns the effective user of the process      #IZ96646
   # where other platforms we support return the real user. Because     #IZ96646
   # we are using this to control whether/how we stop and restart any   #IZ96646
   # agent processes, we need the real user. The new cinfo (with        #IZ96646
   # IZ96646) returns the real user, but older onces do not. So, this   #IZ96646
   # function modifies the output from cinfo as needed.                 #IZ96646
   #--------------------------------------------------------------------#IZ96646
   typeset x y z;                           # local work variables      #IZ96646
   ${CANDLEHOME}/bin/cinfo -r | grep '\.\.running' | while read x; do   #IZ96646
      set -A y ${x};                        # break words into array    #IZ96646
      if [[ "${thisOS}" = "SunOS" ]]; then  # get real userid on SunOS  #IZ96646
         y[3]=$(ps -p ${y[2]} -o ruser | tail +2 | tr -d ' ');          #IZ96646
      fi;                                   # (if(thisOS="SunOS"))      #IZ96646
      z=$(( ${#y[@]} - 2 ));                # penultimate word position #IZ96646
      print -- "${y[0]}|${y[1]}|${y[2]}|${y[${z}]}|${y[3]}";            #IZ96646
   done;                                    # (while(read x))           #IZ96646
}                                           # (cinfoRunning())          #IZ96646


# See if there is CandleManage running in the background, and warn the user if so.
# @41

checkCandleManage()
{
  UNIX95= ps -eo ppid,args | fgrep "$CANDLEHOME/JRE/" | awk '{print $1}' | fgrep -x "`UNIX95= ps -eo pid,args | fgrep '/CandleManage' | awk '{print $1}'`" > /dev/null 2>&1
  if [[ $? -eq 0 ]]
  then
    # Check for silent mode
    if [ -n "$silentMode" ]
    then
      #$NLS-E$ "The Manage Tivoli Enterprise Monitoring Services is running in the background"
      Err_report warn "`nls_replace 'KCI1178W'`"
    else
      #$NLS-E$ "\nThe Manage Tivoli Enterprise Monitoring Services is running in the background, please close it before continuing.  \c"
      print "\n`nls_replace 'KCI1179I'`  \c"

      repeat=1
      while [ $repeat -eq 1 ]; do
        #$NLS-E$ "\nContinue with this installation [ y or n; \"y\" is default ]?  \c"
        print "\n`nls_replace 'KCI1054I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"1\" `nls_replace 'KCI0199I'` ]?  \c"

        read
        [ -z "$REPLY" ] && REPLY=1
          case $REPLY in
    
            n*|N*|2)
              cleanUp 0  # Exit.
              repeat=0
            ;;

            y*|Y*|1)
              repeat=0 
            ;;
          esac
        done
    fi
  fi
}

# Checking soft limit of a maximum number of open files - only for AIX
checkMaxOpenFilesLimit(){
	if [ "$(uname)" = "AIX" ] ; then 
		curOpenFiles=$(( $(lsof -p 0 2> /dev/null | wc -l) + $(lsof -p ^0 2> /dev/null | wc -l) ))
		softLimit=$(ulimit -n 2> /dev/null)

		if [[ $softLimit = +([0-9]) && $(( $curOpenFiles+1000 )) -ge $softLimit ]] ; then
			#$NLS-E$ "Current limit of a maximum number of open files is too small. It is recommended to increase this limit by 1000 as installation may fail."
    		Err_report warn "`nls_replace 'KCI1359W'`"

			if [ -z "$PARM_FILE" ] ; then			
				repeat=1
				while [ $repeat -eq 1 ]; do
					#$NLS-E$ "\nContinue with this installation [ y or n; \"y\" is default ]?  \c"
					print "\n`nls_replace 'KCI1054I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"1\" `nls_replace 'KCI0199I'` ]?  \c"

					read
					[ -z "$REPLY" ] && REPLY=1
					case $REPLY in
						n*|N*|2)
							cleanUp 0  # Exit.
							repeat=0
						;;

						y*|Y*|1)
							repeat=0 
						;;
					esac
				done
			fi
		fi
	fi 
}

# See if there is a CMS or if there are any agents that are still running.  If so, warn the user.
# @11

checkProcesses()
{
  #set -x
  typeset pFile=$CANDLEHOME/registry/proddsc.tbl
  typeset rFile=$CANDLEHOME/config/.ConfigData/RunInfo
  typeset PID desc  msg pc proc 
  
  [ -r $rFile ] || return  # It must be a new install if there is no RunInfo file.
  
  # Determine whether it's BigK or dinkySQL.
  
  fieldSep=$(grep "FIELDSEP" $rFile | cut -d "=" -f2 2> /dev/null)
  
  # See if there are any agents/CMS's running from this <candlehome> on this machine.

  fld123=$(cinfoRunning)  # fld123 -> HOST|ms|35594|TEMS|itmuser
  fld23=$(print "$fld123" | cut -d "$fieldSep" -f2-5)  # fld23 -> ms|35594|TEMS|itmuser
  [ -n "$fld23" ] || return

  
  # Check to see if the processes are actually running.

  abort="N"

  for proc in $fld23
  do
    pc=$(print "$proc" | cut -d "$fieldSep" -f1)
    PID=$(print "$proc" | cut -d "$fieldSep" -f2)
    userid=$(print "$proc" | cut -d "$fieldSep" -f4)


    if [[ "$thisUser" != "root"  && "$thisUser" != "$userid" ]]
    then
      #$NLS-E$ "The userid associated with the $pc product code and $PID process id does not match the current user"
      Err_report warn "`nls_replace 'KCI0244I' \\"$pc\\" \\"$PID\\"`"
      abort="Y"
      continue
    fi

    if [ -z "$msg" ]
    then
	#$NLS-E$ "\\nThe following processes are currently running: \\n"
	msg="\\n`nls_replace 'KCI1049I'` \\n"
    fi
  done


  # IZ17921 check to see if we should abort

    if [[ $abort = "Y" ]]
    then
	#$NLS-E$ "KCI0245E A running process(s) was detected whose associated userid does not match the current userid.  The install will be aborted"
	Err_report fail "`nls_replace 'KCI0245E'`"
    fi

  if [ -n "$msg" ]
  then
    print "$msg"

    for proc in $fld23
    do
      pc=$(print "$proc" | cut -d "$fieldSep" -f1)
    
      # Get the product description from the proddsc.tbl file.  If there are running processes,
      # then this file should exist.
    
      if [ -r $pFile ]  
      then
        desc=$(grep "^$pc|" $pFile |head -1 | cut -d"|" -f2)  # desc -> "NetView/AIX Alert Adapter"
        [ -n "$desc" ] || desc=$pc  # Just in case.
      fi
    
      PID=$(print "$proc" | cut -d "$fieldSep" -f2)
    
	  #$NLS-E$ "  Product = $desc  PID = $PID"
      print "  `nls_replace 'KCI1050I'` = $desc  PID = $PID"
    done
  
    # Check for silent mode
    if [ -n "$silentMode" ]
    then
      # See if they ask to ignore the running proceses (looking for something like "INSTALL_IGNORE_RUNNING_PROCESSES=YES")
      [ -f "$PARM_FILE" ] && continueInstall=$(grep -i "INSTALL_IGNORE_RUNNING_PROCESSES" $PARM_FILE 2>/dev/null | grep -i "y" 2>/dev/null)

      # Warn and exit if they didn't ask to ignore running processes
      if [ -z "$continueInstall" ]
      then
	    #$NLS-E$ "Silent install aborting due to running processes detected on this CandleHome."
        Err_report msgs "`nls_replace 'KCI1051I'`"
        cleanUp 1	# Exit - cc=1
      fi

      # Otherwise just continue with the install
	  #$NLS-E$ "Silent install option to ignore running processes detected"
      Err_report warn "`nls_replace 'KCI1052I'`"
      cd $CANDLEHOME/bin
      restart_proc stop
	  restart_proc_rc=$?
	  if [[ $restart_proc_rc != 0 ]]
	  then
	    #$NLS-E$ "Silent install aborting due to stopping TEMS failure. Chcek logs for more details."
        Err_report msgs "`nls_replace 'KCI1232I'`"
		#$NLS-E$ "You can try to manually stop TEMS and again run installation."
		Err_report fail "`nls_replace 'KCI1234I'`"
	  fi 
      cd -
      return
    fi


	repeat=1
	while [ $repeat -eq 1 ]; do
		#$NLS-E$ "Existing products found to be running will be restarted during installation."
    	Err_report warn "`nls_replace 'KCI1053I'`"
		#$NLS-E$ "\nContinue with this installation [ y or n; \"y\" is default ]?  \c"
    	print "\n`nls_replace 'KCI1054I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"1\" `nls_replace 'KCI0199I'` ]?  \c"
    
    	read
    	[ -z "$REPLY" ] && REPLY=1
    	case $REPLY in
    		
      		n*|N*|2) cleanUp 0  # Exit.
      			repeat=0
	    	 ;;
    
          	y*|Y*|1)
          	    repeat=0 
     			cd $CANDLEHOME/bin
     			restart_proc stop
				restart_proc_rc=$?
				if [[ $restart_proc_rc != 0 ]]
				then
					#$NLS-E$ "Install aborting due to stopping TEMS failure. Chcek logs for more details."
					Err_report msgs "`nls_replace 'KCI1233I'`"
					#$NLS-E$ "You can try to manually stop TEMS and again run installation."
					Err_report fail "`nls_replace 'KCI1234I'`"
				fi 
     			cd -
             ;;
    	esac
	done
    echo
  fi
}

# Warn user about the dangers of installing over an old Roma

BspWarn()
{
  #$NLS-E$ "****************** WARNING ***************************"
  Err_report msgs "`nls_replace 'KCI1055I'`"
  #$NLS-E$ "You are installing eBP over an existing BSP $LDAPVer"
  #$NLS-E$ "installation.   This will migrate your LDAP to the"
  #$NLS-E$ "newer version,  and requires your LDAP server to be"
  #$NLS-E$ "shut down.  Please ensure that there are no remote"
  #$NLS-E$ "BSP applications  using this server.  If this file"
  #$NLS-E$ "system is accessible from other systems,  please"
  #$NLS-E$ "ensure that there are no applications running on"
  #$NLS-E$ "other systems from this CandleHome."
  
  Err_report msgs "`nls_replace 'KCI1056I' $LDAPVer`"  
  Err_report msgs "`nls_replace 'KCI1057I'`"
  Err_report msgs "`nls_replace 'KCI1058I'`"
  Err_report msgs "`nls_replace 'KCI1059I'`"
  Err_report msgs " "
  #$NLS-E$ "Please stop your LDAP server now if it's currently running."
  Err_report msgs "`nls_replace 'KCI1060I' LDAP`"
  #$NLS-E$ "****************** WARNING ***************************"
  Err_report msgs "`nls_replace 'KCI1055I'`"
  sleep 5
}

# Routine renames obsolete Roma 2.1 directory names for a BSP 3.1 install
# This entire routine can be removed when 2.1 is no longer supported.

Bsp21Rename()
{
  #set -x
  RegBackup=$1

  # We don't do any renames except for a prior BSP 2.1
  [ -r "$RegBackup/Pre_CT99_Roma" ] || return

  oldName=$(cat $RegBackup/Pre_CT99_Roma|head -1)
  oldPath=${oldName%/ldap}
  oldArch=${oldPath#$CANDLEHOME/roma/}
  typeset thisOS=$(uname -s)
  #
  # Roma directory remap from BSP 2.1->3.1
  # 2.1	-> 3.1
  # aix4 	-> aix42
  # aix43 	-> aix42
  # hp10 	-> hp11
  # hp102 	-> hp11
  # sol25 	-> sol26
  # sol273 	-> sol26

  # New install paths for BSP 3.1
  [ "$thisOS" = "AIX" ] && newPath=$CANDLEHOME/roma/aix42
  [ "$thisOS" = "HP-UX" ] && newPath=$CANDLEHOME/roma/hp11
  [ "$thisOS" = "SunOS" ] && newPath=$CANDLEHOME/roma/sol26

  # If the new path already exists as a file, rename it
  [ -f $newPath ] && mv $newPath ${newPath}__RenamedFile

  # If the old path matches the new one, we're done
  [ "$oldPath" = "newPath" ]  && return

  # Rename any (known) old path to the new one
  [ "$oldArch" = "aix4" ]  && mv $oldPath $newPath
  [ "$oldArch" = "aix43" ] && mv $oldPath $newPath
  [ "$oldArch" = "hp10" ]  && mv $oldPath $newPath
  [ "$oldArch" = "hp102" ] && mv $oldPath $newPath
  [ "$oldArch" = "sol25" ]  && mv $oldPath $newPath
  [ "$oldArch" = "sol273" ] && mv $oldPath $newPath

  # Reset the tag file to the new name
  find $newPath -name ldap -print|head -1 > $RegBackup/Pre_CT99_Roma
}

# Display the license text in command-line mode

shoPrequis()
{
#$NLS-E$ "\n    Before installing IBM Tivoli Monitoring agents, you must"
#$NLS-E$ "install at least one IBM Tivoli Enterprise Monitoring Server.  You"
#$NLS-E$ "will need the host name or IP address and port number for the"
#$NLS-E$ "monitoring server to configure any agents.\n"
echo
nls_replace 'KCI1061I'  
nls_replace 'KCI1062I'  
echo
sleep 5 
}

shoLicense()
{
  which nroff  2> /dev/null > /dev/null  # See if nroff is available
  if [ $? = 0 ]                          #  we just do "more" if it's not
  then
    nroff $license | more -s
    print ""
  else
    more -s $license
  fi
  print " "
}

getVFNames()
{
  typeset -i i=0
  typeset arch fullName name pc verName

  ls $CANDLEHOME/registry/*.ver > /dev/null 2>&1
  [ $? -eq 0 ] || return  # Nothing to process.

  for name in $(ls $CANDLEHOME/registry/*.ver)
  do
    verName=${name##*/}  # Remove the pathname.
    pc=$(print $verName | cut -c1-2 2> /dev/null)   # Get the product code.
    [[ $pc = @(cz|or|ox|oy|ul|um|ux) ]] || continue   # Only a few products need special permissions.

    arch=$(print $verName | cut -c3- 2> /dev/null)  # Determine the bin arch.
    [[ $arch = @(aix|hp|sol)* ]] || continue  # Skip CMS data pieces.

    vfNames[i]=$verName
    i=i+1
  done

  procNum=${#vfNames[*]}
}

# find proper GSKit package
# $1 temporary catalog for untar purpose
# $2 path to directory with GSKit images
# $3 architecture searched
# return (on success)
# $pgskTar - tar file name with proper image
# $pgskVer - version file name
# $gskPath - gs directory
# return (on failure)
# $pgskTar empty

find_gskit_package() {
  #set -x
  typeset tempdir=$1
  typeset path=$2
  typeset arch=$3
  
  tmpFile=$tempdir/$$
  removeFile $tmpFile
  typeset -i numb=0

  gskPgks=$(ls $path)
  for p in $gskPgks; do
    tarName=$p
    baseName=${p%.*}
    verName=$baseName.ver
    pathName=${baseName##*gs}
    tarPath=$path/$tarName
    if test -d $tarPath; then continue; fi
    getVerFromTar $tarPath $verName $tempdir
    if [ -z "$tmpVerFile" ]; then
      Failure_report "Cannot continue installation"  
      return;
    fi
    
    foundSth=$(grep -l "^runArch *= *$arch" $tmpVerFile 2> /dev/null)
    if [ -n "$foundSth" ]
    then
      pgskTar=$tarName
      pgskVer=$verName
      pgskPath=$pathName
      numb=numb+1
      echo $tarPath >>$tmpFile
      # check for prereq
      getPropertyValue $tmpVerFile preReqALL
      # get prereq platform from prereqAll property
      
      # Defect 70794, 2008/05/30, does not work on Linux
      # print $resultProperty | cut -f 1 -d \|  | read prereqValue
      prereqValue=$(print "$resultProperty" | cut -f 1 -d \|)

      if [ -n "$prereqValue" ] && [ "$prereqValue" != "none" ]; then
        pgskTar1=$prereqValue.ctp
        [[ -f $path/$pgskTar1 ]] || pgskTar1=$prereqValue.tar
        pgskVer1=$prereqValue.ver
        pgskPath1=$(print $prereqValue | cut -c3-)
        # check if tar file exists
        checkForFile $path/$pgskTar1
      fi	  
    fi
    removeFile $tmpVerFile
  done
  
  if [ $numb -eq 0 ]; then removeFile $tmpFile; return; fi
  if [ $numb -eq 1 ]; then removeFile $tmpFile; return; fi
  Log_report "More than one ($numb) package in $path directory matches $arch architecture"
  Log_report "List of matching gs packages found"
  Log_report "($tmpFile)"
  more $tmpFile
  Failure_report "Cannot continue installation"  
}
	

# set gskArch directory and gskTar file name specific for platform
# $gskVer - version file
# set alternatively also gskArch64 and gskTar64
# uses variables: $os, $bitNum, $hw, $cArch
setGSKtarnames() {
  #set -x
  typeset tempDirectory=/tmp/itm$$
  unset pgskTar
  unset pgskVer
  unset pgskPath
  unset pgskTar1
  unset pgskVer1
  unset pgskPath1
  # if product directory exitst
  if [ ! -d "$candleGSKitProdDir" ] ; then
    Log_report "No $candleGSKitProdDir - assuming GSKit already installed."
    checkForFile $candleGskitgskInfoFile
    return
  fi
  
  removecreateDir $tempDirectory
#"Searching for proper GSKit package in $candleGSKitProdDir directory"
  Log_report "`nls_replace 'KCI1323I' \\"$candleGSKitProdDir\\"`"
  find_gskit_package $tempDirectory $candleGSKitProdDir $cArch
  removeDirectory $tempDirectory
  if [ -z "$pgskTar" ]; then 
    nosupportedPlatform $cArch $gskitDir
    return
  fi
  if [ -n "$pgskTar1" ] ; then 
    gskArch64=$pgskPath
    gskTar64=$pgskTar
    gskVer64=$pgskVer
    gskArch=$pgskPath1
    gskTar=$pgskTar1
    gskVer=$pgskVer1
  else 
    gskArch=$pgskPath
    gskTar=$pgskTar
    gskVer=$pgskVer
  fi  
}  
    
exportGskitVariables() {
  export gskArch
  export gskTar
  export gskVer
  export gskArch64
  export gskTar64
  export gskVer64
  export PROD_PATH
  export AWK
}  

setSomeCandleEnvironment() {
  oldCandleGskitDirectory=$CANDLEHOME/gskit
  candleGskitRunTimeDirectory=$CANDLEHOME
  candleGskitgskInfoFile=$CANDLEHOME/config/gsKit.config
  CANDLETMP=$CANDLEHOME/tmp
  candleGSKitProdDir=$PROD_PATH/unix/GSKit
}

removecreateDir() {
  if test -d $1; then rm -rf $1; fi
  mkdir $1
}

removeFile() {
  if test -f $1; then 
    rm $1 || couldNotRemoveFile $1 
  fi
}

Log_report() {
  Err_report info "$1"	
}

# Extracts ver file from tar file
# $1 - path to tar file
# $2 - ver file name to extract
# $3 - temporary directory for use
# example getVerFromTar {...}/GSKit/gsli6243.tar gsli6243.ver /tmp
# output:
# success: tmpVerFile=constains ver file
# failure: tmpVerFile empty

getVerFromTar() 
{
   typeset ltarPath=$1
   typeset lverName=$2
   typeset ltempdir=$3

   unset tmpVerFile   
   CMD="unloadTmpFile $lverName $ltarPath"      
   runCommandPerfectDirectory $ltempdir $CMD 
   if [ $? -ne 0 ] ; then
      packageVerError tmp/$lverName $ltarPath
      return
   fi
   tmpVerFile=$ltempdir/tmp/$lverName
}

runCommandPerfectDirectory() {
  typeset DIR=$1
  shift 
  typeset cmd=$*
  typeset ADIR=$PWD
  cd $DIR
  runCommandPerfect $cmd
  typeset cc=$?
  cd $ADIR
  return $cc
}

runCommandPerfect() {
   typeset cmd=$*
#  Log_report "cmd: $cmd"
   typeset cc
   eval $cmd
   cc=$?
   if [ $cc -gt 0 ]; then
      Failure_report "Return error code: $cc"
   fi
   return $cc
}

nosupportedPlatform() {
	#$NLS-E$ "GSK installation error: platform $1 not supported, no proper tar file in $2 directory"
  Failure_report "`nls_replace 'KCI1249E' \\"$1\\" \\"$2\\"`"
}

# Get property value from file 
# $1 file to get property
# $2 property
# on success : resultProperty holds value
# on failure: empty value

getPropertyValue() {
     unset resultProperty
     if line=$( grep "^$2 *= *" $1); then
       resultProperty=${line##*=}
       # remove trailing spaces
       resultProperty=${resultProperty#\ *}
     fi
}

removeDirectory() {
  if test -d $1; then
     rm -rf $1
  fi
}

checkGSKtarnames() {
  if [ -z "$gskArch" ] || [ -z "$gskTar" ] || [ -z "$gskVer" ] ; then gskSupportError; fi
  if [ -n "$gskArch64" ] && [ -z "$gskTar64" ] ; then gskSupportError; fi
}        

checkForFile() {
   test -f $1
   if [ $? -ne 0 ]; then 
     packageError $1
   fi
}

packageError() {
  Failure_report "PACKAGE ERROR: no file $1"
}

Failure_report() {
  Err_report fail "$1"
}

# check if iconv exists in the system 
checkIconv()
{
  iconv_path="$(whence iconv 2>/dev/null)"
  if [ -z "$iconv_path" ]
  then
    # iconv does not exist in the system.
    # This message should not be globalised.
    print "************************* WARNING *************************"
    print "The program iconv is not available and should be installed."
    print "************************* WARNING *************************"
  fi
}

# This cleans up the system before and after we run the entire install
# - mainly used to clean our /tmp
cleanupSystem()
{

  # Our plugin work dir
  rm -rf /tmp/plugin_tmp > /dev/null 2>&1

  #on HP-IA, the temporary directory is /var/tmp
  if [ -d /var/tmp/plugin_tmp ]
  then
      rm -rf /var/tmp/plugin_tmp > /dev/null 2>&1
  fi

}

checkSYHDupgrade()
{
  if [ "$1" = "post" ]
  then  	
  	if [ -z "$syNotInstalled" ]
  	then
      syVerFile=$(ls $curRegPath/sy*.ver 2>/dev/null | grep -v syt.*ver)  
      if [ -f "$syVerFile" ]
      then
        # Read ver, rel and VRMF. Truncate space and 0
        newSYver=$(grep "^ver *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
        newSYver=${newSYver##+( )+(0)}
        newSYrel=$(grep "^rel *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
        newSYrel=${newSYrel##+( )+(0)}
        newSYvrmf=$(grep "^VRMF *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
        newSYvrmf=${newSYvrmf##+( )+(0)}
        
        oldSYverTrim="$(echo $oldSYver | sed 's/^[ 0]+//')"
        
        # not in deploy or in deploy and oldVer=610
        [[ $DEPLOY_MODE = "" || "$oldSYverTrim" = "610" ]] && [[ "$newSYver" != "$oldSYver" || "$newSYrel" != "$oldSYrel" || "$newSYvrmf" != "$oldSYvrmf" ]] && syNotStart=yes
      fi
    fi   

    if [ -z "$hdNotInstalled" ]
    then
      hdVerFile=$(ls $curRegPath/hd*.ver 2>/dev/null | grep -v hdt.*ver)
      if [ -f "$hdVerFile" ]
      then
        # Read ver, rel and VRMF. Truncate space and 0
        newHDver=$(grep "^ver *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)
        newHDver=${newHDver##+( )+(0)}
        newHDrel=$(grep "^rel *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)
        newHDrel=${newHDrel##+( )+(0)}
        newHDvrmf=$(grep "^VRMF *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)        
        newHDvrmf=${newHDvrmf##+( )+(0)}
        
        oldHDverTrim="$(echo $oldHDver | sed 's/^[ 0]+//')"
        
        # not in deploy or in deploy and oldVer=610
        [[ $DEPLOY_MODE = "" || "$oldHDverTrim" = "610" ]] && [[ "$newHDver" != "$oldHDver" || "$newHDrel" != "$oldHDrel" || "$newHDvrmf" != "$oldHDvrmf" ]] && hdNotStart=yes
      fi  
    fi      
  fi
  
  if [ "$1" = "pre" ]  
  then
    syVerFile=$(ls $curRegPath/sy*.ver 2>/dev/null | grep -v syt.*ver)
    if [ -f "$syVerFile" ]
    then
      # Read ver, rel and VRMF. Truncate space and 0
      oldSYver=$(grep "^ver *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
      oldSYver=${oldSYver##+( )+(0)}
      oldSYrel=$(grep "^rel *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
      oldSYrel=${oldSYrel##+( )+(0)}
      oldSYvrmf=$(grep "^VRMF *=" $syVerFile 2>/dev/null | cut -d"=" -f2)
      oldSYvrmf=${oldSYvrmf##+( )+(0)}
      
    else
      syNotInstalled=yes  
    fi          
    hdVerFile=$(ls $curRegPath/hd*.ver 2>/dev/null | grep -v hdt.*ver)
    if [ -f "$hdVerFile" ]
    then
      # Read ver, rel and VRMF. Truncate space and 0
      oldHDver=$(grep "^ver *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)      
      oldHDver=${oldHDver##+( )+(0)}
      oldHDrel=$(grep "^rel *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)        
      oldHDrel=${oldHDrel##+( )+(0)}
      oldHDvrmf=$(grep "^VRMF *=" $hdVerFile 2>/dev/null | cut -d"=" -f2)  
      oldHDvrmf=${oldHDvrmf##+( )+(0)}
    else
      hdNotInstalled=yes
    fi  
  fi    
}

# This function is used to determine if the ITM installer is running on a supported platfrom.
# The platform is supported when there is a JRE for the current architecture and a appropriate Gskit package.
# If the ITM installer is running on supported platform, then  "0" is returned.
# f74525
isPlatformSupported()
{
  #set -x
  # @40 check for Application Support Files only install
  if [ "$PLATFORM_SUPPORTED_CHECK" = "NO" ]; then
    # Extract the .ver file for this new media installer
    unloadTmpFile $ciVFile $PKGLOC/$ciTar  
    mediaFile=$CANDLEHOME/tmp/$ciVFile
    mediaVer=$(grep "^ver *=" $mediaFile 2> /dev/null | cut -d"=" -f2)
    # Also get the version for the current installer
    currentFile=$CANDLEHOME/registry/$ciVFile
    currentVer=$(grep "^ver *=" $currentFile 2> /dev/null | cut -d"=" -f2)
    if [[ "$currentVer" -ge "$mediaVer" ]]; then
    # At this point, since only Application Support Files are being installed, no platform support checks should be performed.
      return 0  
    else
      # d94520 We are tring to install in a directory that does not have a current ITM environment (either no environment or version is backlevel from this media)
      # Display an appropriate message so user can correct this, rather than letting it fall through and getting a generic message.
      # $NLS-E$ "KCI1292I Installations from application support media must be installed into a directory containing an existing IBM Tivoli Monitoring environment."
       Err_report fail "`nls_replace 'KCI1292I'`"
    fi
    
  fi
  arch=$1 #li6242, li6266, aix533 ect.
  tmpDir=$CANDLEHOME/tmp
  jreOS=${arch%%[0-9]*} #aix, li  
  cd $tmpDir
  if [ $? -eq 0 ]
  then
    rm -rf tmp > /dev/null 2>&1
  else
    # $NLS-E$ "\n\"$CANDLEHOME\tmp" does not exist"
    Err_report fail "`nls_replace 'KCI1069I' \\"$tmpDir\\"`"
  fi
  
  # Copy the jr<arch>.ver files from the media to the $CANDLEHOME/tmp/tmp directory if they exist.
  # Otherwise,  assume they were extracted in cienv.tar and
  # copy all of the jr*.ver files from cienv.tar to the $CANDLEHOME/tmp/tmp directory.

  jreOS=${cArch%%[0-9]*} #aix, li, sol, hp, etc
  [ -d tmp ] || mkdir tmp > /dev/null 2>&1
  
  for file in `ls $PKGLOC/jr$jreOS*.tar $PKGLOC/jr$jreOS*.ctp 2> /dev/null`
  do
    pkgName=$(basename $file)
    pkgName=${pkgName%.*}
    if [ -f $PKGLOC/$pkgName.ver ]
    then
      cp $PKGLOC/$pkgName.ver tmp/$pkgName.ver > /dev/null 2>&1
    else
	  tar -$tarOpt $PKGLOC/cienv.tar tmp/$pkgName.ver > /dev/null 2>&1	
	fi
  done
  # Check if there is supported JRE for current arch
  export LANG=C
  jre=$(grep -l "^runArch *= *$arch" tmp/jr$jreOS*.ver 2> /dev/null)  	
  export LANG=$saveLang  
  if [[ -z "$jre" ]] #if no JRE found
  then
    rm -rf tmp > /dev/null 2>&1
    cd $CANDLEHOME
    return 1 #no jre found, platform not supported	
  fi 
  rm -rf tmp > /dev/null 2>&1
  for file in `ls $PKGLOC/GSKit/gs$jreOS* 2> /dev/null`
  do
    pkgName=$(basename $file)
    pkgName=${pkgName%.*}
	unloadTmpFile $pkgName.ver $file
  done
  # Check if there is GSkit package for current arch
  export LANG=C
  gskit=$(grep -l "^runArch *= *$arch" tmp/gs$jreOS*.ver 2> /dev/null)
  export LANG=$saveLang  
  rm -rf tmp > /dev/null 2>&1
  cd $CANDLEHOME
  [[ -z "$gskit" ]] && return 2 #no gskit found, platform not supported	
  return 0
}

# Check if a platform is supported before installing anything.
# Unpack files to the CANDLEHOME/tmp directory and run validation.
# It is assumed that $CANDLEHOME is the current directory.

checkPlatformBeforeUpgrade() 
{
    #set -x
    cd $CANDLETMP 
    tar xvf $PKGLOC/cienv.tar tmp/archdsc.tbl >/dev/null 2>&1
    tar xvf $PKGLOC/cienv.tar bin >/dev/null 2>&1
    . $CANDLETMP/bin/dynarch.shl $CANDLETMP/tmp/archdsc.tbl
    cd $CANDLEHOME
    rm -rf $CANDLETMP/tmp 
    rm -rf $CANDLETMP/bin

    if [ -n $cArch ]
    then
        isPlatformSupported $cArch
        [[ $? -ne 0 ]] && Err_report fail "`nls_replace 'KCI1148E'`" 
        platformChecked="YES"
    fi
}

#block upgrade from releases older than ITM6.1
blockOmegamonUpgrade()
{
  export LANG=C
  productsList="cq ms cj sy hd lz ux um"
  for product in $productsList
  do
    for file in `ls $CANDLEHOME/registry/$product*.ver 2>/dev/null | grep $pc[a-z]*[0-9][0-9]*\.ver`
    do
      cVer=$(grep "^ver *=" $file 2>/dev/null | cut -d"=" -f2)
      [[ "$cVer" -lt 610 ]] && Err_report fail "`nls_replace 'KCI1256E'`"
    done
  done
  export LANG=$saveLang
}

validateSecureGroup()
{
  # first check if secureMode file exists
  secureFile=$CANDLEHOME/registry/SecureMode
  if [[ -f "$secureFile" ]];
  then
    line=$(cat $secureFile 2>/dev/null)
    mode=$(print $line | cut -f1 -d:)
    group=$(print $line | cut -f3 -d:)
    if [[ -n "$group" && "$group" != "$SECURE_GROUP" ]];
    then
      # $NLS-E$ KCI1347E Group $1 is different than common group $2 previously used to secure IBM Tivoli Monitoring environment.
      Err_report fail "`nls_replace 'KCI1347E' $SECURE_GROUP $group`"
    fi
  fi  
  
  # verify if group name is valid and user is a member of SECURE_GROUP
  testFile=/tmp/secureTest.$$
  touch "$testFile" > /dev/null 2>&1
  if [[ $? = 0 ]];
  then
  	# testFile created without any problem, use chgrp to validate group
    chgrp "$SECURE_GROUP" "$testFile" > /dev/null 2>&1
    rcGroup=$?
    rm -f "$testFile" > /dev/null 2>&1
    [[ "$rcGroup" = 0 ]] && valid=yes
  else
    # use alternative method when testFile can't be created, this check is only for non root user
    [[ "${thisRealUser}" = "root" ]] && [[ -n "$(groups | grep -w $SECURE_GROUP)" ]] && valid=yes #IZ94626.1
  fi  
  
  [[ -z "$valid" ]] && Err_report fail "`nls_replace 'KCI1346E' $SECURE_GROUP $thisUser`" # $NLS-E$ The specified group $1 is not a valid group or user $2 is not a member of this group.            
}
  
###################################################################
#                          Main                                   #
###################################################################
#set -x
export saveLang=$LANG

bindir=`pwd`/$(dirname $0)
[ "$(print $0 | grep '\./')" = "" ] && [ "$(print $0 | grep '^/.*')" = "" ] && [ `pwd` = '/' ] && bindir=`pwd`$(dirname $0)
[ "$(print $0 | grep '^/.*')" != "" ] && bindir=$(dirname $0)

 

if [ -d $bindir/unix ] 
then
nls_replace_dir=$bindir/unix/nls
else
nls_replace_dir=$bindir
fi
export PATH=$nls_replace_dir:$PATH

export PATH=/usr/bin:$PATH
UsrCmd="$0 $*"
PROGNAME=$(basename $0)
PROGDIR=$(dirname $0)

cd "$PROGDIR"

typeset -i i=1 idx=0 j=0 kbAvail=0 kbTmpAvail=0 kbReq=0 kbytes=0 numEle=0 numItems=0
typeset curJVF="" ext="" newJVF="" osn="" toJVF="" AWK="awk"
typeset hasCCC="" hasFixes=false hasReadmes=false hasRoma="" hasCASP="" hasLDAP="" hasCMS=""
typeset hadCCC="" hadRoma="" hadCASP="" hasLDAP=""
typeset parDir prodMode tarDir  # Used by CandleRemote.
typeset license="license.txt"
typeset fld23 fld123 fieldSep
typeset hostNameInRunInfo
typeset platformChecked="NO"

typeset baseTok thisMach thisRec thisTok thisUser  # Variables used for the token file.

set -A mainMenu
set -A mainMenu2


CDNAME=GWV510
export CDNAME
AUTHKEY=2be4eceb
# @40 This is the trigger for logic check in isPlatformSupported routine:
#   - Set to NO if the install is for media containing Application Support files only
#   - Unset ( or keep original setting as __PLATFORM_SUPPORTED_CHECK__ ) for base and agent media install
PLATFORM_SUPPORTED_CHECK=NO
DEBUG_CON=""
SAFE_MODE=""
silentMode=""
PARM_FILE=""
CLI_OPTION="installProduct"
DEPLOY_MODE=""
SUSE_VERSION=""
SECURE_GROUP=""
ciTar=cienv.tar  # NOTE:  this file is not compressed.
ciVFile=cienv.ver
instCI=no instCIFix=no instJRE=no
instPkgs="ci*.tar jr*.tar jr*.ctp jr*.z* *.jar *.j0*"
oldJRE=jre1.1.6
reqDirs="JRE bin config doc download logs misc registry tables tmp xfer keyfiles"
tables="archdsc.tbl imdprof.tbl imdver.tbl proddsc.tbl" 

typeset thisOS=$(uname -s)

#-----------------------------------------------------------------------#IZ94626.1
# Use the id command to get our real and effective user names.          #IZ94626.1
#-----------------------------------------------------------------------#IZ94626.1
if [[ "${thisOS}" = "SunOS" ]]; then                                    #IZ94626.1
   id_cmd="/usr/xpg4/bin/id";                                           #IZ94626.1
else                                                                    #IZ94626.1
   id_cmd="id";                                                         #IZ94626.1
fi;                                                                     #IZ94626.1
thisRealUser=$(${id_cmd} -unr);             # get real user name        #IZ94626.1
thisEffectiveUser=$(${id_cmd} -un);         # get effective user name   #IZ94626.1
thisUser="${thisEffectiveUser}";            # generally, we want euid   #IZ94626.1
                                                                        #IZ94626.1
#-----------------------------------------------------------------------#IZ94626.1
# If our real userid is root, then we can use the su command. Note      #IZ94626.1
# that it doesn't matter if our effective userid is root. The AIX and   #IZ94626.1
# HP-UX versions of su determine authorization based on real userid,    #IZ94626.1
# and will fail if our real userid is not root.                         #IZ94626.1
#-----------------------------------------------------------------------#IZ94626.1
if [[ "${thisRealUser}" = "root" ]]; then                               #IZ94626.1
   su_value=$(locateCMD su);                                            #IZ94626
else                                                                    #IZ94626.1
   su_value="";                                                         #IZ94626.1
fi;                                                                     #IZ94626.1
                                                                        #IZ96646.1
#-----------------------------------------------------------------------#IZ96646.1
# On Solaris, if the real and effective user are not the same, we set   #IZ96646.1
# umask 000 so plugin processing doesn't run into permission issues     #IZ96646.1
# with the system temporary directory.                                  #IZ96646.1
#-----------------------------------------------------------------------#IZ96646.1
if [[ "${thisOS}" = "SunOS" ]]; then                                    #IZ94626.1
   if [[ "${thisRealUser}" != "${thisEffectiveUser}" ]]; then           #IZ94626.1
      umask 000;                                                        #IZ96646.1
   fi;                                                                  #IZ94626.1
fi;                                                                     #IZ94626.1

# need to use nawk if it's available
type nawk >/dev/null 2>&1
[[ $? -eq 0 ]] && AWK=nawk

# check if iconv exists in the system 
checkIconv

while getopts ":h:j:d:o:p:k:acqvg" opt
do
  case $opt in
    a) DEPLOY_MODE="-Ditm.deploy=true"
       ;;
    h) CANDLEHOME=$OPTARG
       CANDLEHOME=$(eval print $CANDLEHOME)  # Process ~ if it was entered.
       ;;
    j) JAVALOC=$OPTARG
       JAVALOC=$(eval print $JAVALOC)  # Process ~ if it was entered.
       ;;
    d) PROD_PATH=$OPTARG
       PROD_PATH=$(eval print $PROD_PATH)
	   nls_replace_dir=$PROD_PATH/unix/nls
       export TEXTDOMAINDIR=$nls_replace_dir/msg
	   export PATH=$nls_replace_dir:$PATH
	   ;;
    c) DEBUG_CON="-debugcon"
       ;;
    o) CLI_OPTION="installProduct"
       [ -n "$(print $OPTARG | grep -i "product" | grep -i "depot")" ] && CLI_OPTION="installProductToDepot"
       ;;
    p) PARM_FILE=$OPTARG
       export PARM_FILE
       silentMode="-t 5"
       ;;
    g) GSKIT_DOWNGRADE="yes"
       export GSKIT_DOWNGRADE
       ;;	
    q) silentMode="-t 5"
       VERBOSE="yes" # by default, verbose in silent mode for deploy
       export VERBOSE
       ;;
    k) SECURE_GROUP=$OPTARG
       export SECURE_GROUP
       ;;
    v) VERBOSE="yes"
       export VERBOSE
       ;;
    ?) Use_report
       ;;
    :) Use_report
       ;;
  esac
done

checkMaxOpenFilesLimit

#check if correct version of psksh is installed:
typeset -i number20=0 number19=0
number20=020
number19=019
[ number19 -gt number20 ] && print -u2 "`nls_replace 'KCI1318I'`"

#try to check if scirpt was run with wildcard path and more than 1 path was resolved (defect 71768, 72307)
if [ $# -gt 0 ] && [ -n $1 ] && [ `echo $1 | cut -c1` != "-"  ] && [ 'install.sh' = `basename $1` ]
then
   #$NLS-E$ "  KCI1176E You have probably run install.sh using wildcards in path and this path was resolved to more than one string."
   print -u2 "`nls_replace 'KCI1176W' -c`"
fi

shift OPTIND-1
[ $# -eq 0 ] || Use_report

if [ -n "$DEPLOY_MODE" ]
then
	trap "true" 13 15  # Set trap.
	trap "cleanUp trap" 1 2 3  # Set trap.
else
	trap "cleanUp trap" 1 2 3 15  # Set trap.
fi

# For AIX installations called by Remote Deploy, the PATH is based on the OS
# agent which launchs install.sh.  This PATH does not include /usr/sbin
# therefore it must be added to run slibclean, rmitab and mkitab.
# [[ "$(uname)" = "AIX" ]] && [[ -n "$DEPLOY_MODE" ]] && PATH=$PATH:/usr/sbin
# Adding it always as user may not have /usr/sbin in the PATH
[[ "$(uname)" = "AIX" ]] && PATH=$PATH:/usr/sbin

# In silent mode, make sure all information is known up front
if [ -n "$silentMode" ]
then
  #$NLS-E$ "ITM home directory must be specified in silent mode (-h)"
  [ -z "$CANDLEHOME" ] && Err_report fail "`nls_replace 'KCI1063E' -h`"
  #$NLS-E$ "Input parameter file must be specified in silent mode (-p)"
  [ -z "$PARM_FILE" ] && Err_report fail "`nls_replace 'KCI1064E' -p`"
  #$NLS-E$ "Valid input parameter absolute file path must be specified in silent mode (-p)"
  [ -r /$PARM_FILE ] || Err_report fail "`nls_replace 'KCI1065E' -p`"
fi

welcome="INSTALL"
[[ "$AUTHKEY" = "__AUTHKEY__" ]] && welcome="UPGRADE"
print $welcome
grep "IBM TIVOLI MONITORING (\"ITM\")" ./non_ibm_license

# check write permissions for /tmp directory
[ -w "/tmp" ] || Err_report fail "`nls_replace 'KCI1250E'`"

# check for selinux
if [[ "$thisOS" = "Linux" ]];
then
  eval selinuxenabled >/dev/null 2>&1
  [[ $? -eq 0 ]] && print "\nWARNING: SELinux (Secure Enhanced Linux) is set to \"enforced\" or \"permissive\" mode. \"Enforced\" mode is not supported by IBM Tivoli Monitoring. \"Permissive\" mode may require additional system configuration steps. Perform the following steps to set the mode to \"disabled\" or refer to IBM Tivoli Monitoring documentation for more details:\n1. Set SELinux \"disabled\" mode.\n2. Reboot the system.\n3. Execute the \"prelink -a\" command before restarting the ITM installation."
fi

# Call to cleanup some files we have around
cleanupSystem

# Set the default for CANDLEHOME.

defaultCH=/opt/IBM/ITM

# Determine CANDLEHOME

while true
do
  repeat=true
  if [ -z "$CANDLEHOME" ]
  then
    CANDLEHOME=$defaultCH
	#$NLS-E$ "\nEnter the name of the IBM Tivoli Monitoring directory"
    echo
	nls_replace 'KCI1066I'
	#$NLS-E$ "[ default = $CANDLEHOME ]:  \c"
    print "`nls_replace 'KCI1067I' $CANDLEHOME` \c"
    read resp
    resp=$(eval print $resp)
    [ -n "$resp" ] && CANDLEHOME=$resp
  fi

  if [ ! -d "$CANDLEHOME" ]
  then # The directory $CANDLEHOME does not exist
    # Silent mode defaults to "yes" to create a new CandleHome
    if [ -n "$silentMode" ]
    then  
      mkdir -p $CANDLEHOME > /dev/null 2>&1
      #$NLS-E$ "KCI1236E could not create $CANDLEHOME directory"
      [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1236E' \\"$CANDLEHOME\\"`"
      chmod a-s $CANDLEHOME > /dev/null 2>&1
      #$NLS-E$ "KCI1237E could not chmod for $CANDLEHOME directory"
      [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1237E' \\"$CANDLEHOME\\"`"
      created=true
      repeat=true
    else
      if [ "$(print $CANDLEHOME | wc -w)" -ne "1" ]
      then
	    #$NLS-E$ "\n\"$CANDLEHOME\"  - blanks are not permitted in a directory name."
        echo
		nls_replace 'KCI1068I' \"$CANDLEHOME\"
	created=false
	CANDLEHOME=""
      else
	    #$NLS-E$ "\n\"$CANDLEHOME\" does not exist"
        echo
		nls_replace 'KCI1069I' \"$CANDLEHOME\"
		#$NLS-E$ "Try to create it [ y or n; \"y\" is default ]?  \c"
        print "`nls_replace 'KCI1070I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"1\" `nls_replace 'KCI0199I'` ]?  \c"
        read
        case $REPLY in
          n*|N*|2) exit 0
             ;;
          y*|Y*|1) 
                MKDIRMSG=$(mkdir -p $CANDLEHOME 2>&1|grep -i permission)
		if [ -n "$MKDIRMSG" ]
		then
		    if [ "$(uname)" = "AIX" ]
		    then
			MKDIRMSG=$(print $MKDIRMSG|grep "The file access permissions do not allow the specified action")
		    else
			MKDIRMSG=$(print $MKDIRMSG|grep "Permission denied")
		    fi
			#$NLS-E$ "You don't have permission to create $CANDLEHOME. \nPlease use another directory for IBM Tivoli Monitoring directory."
    		    [ -n "$MKDIRMSG" ] &&  print "`nls_replace 'KCI1071I' $CANDLEHOME` \n `nls_replace 'KCI1072I'`"
		    created=false
		    repeat=false
		else
                    chmod a-s $CANDLEHOME > /dev/null 2>&1
                    created=true
		fi
             ;;
#          thisisbogusresponsetoquestion) Err_report warn "only a tester would enter that!"
#	     created=false
#	     repeat=true
#             ;;
            *)
             if [ "$REPLY" = "" ] #default answer
             then
                MKDIRMSG=$(mkdir -p $CANDLEHOME 2>&1|grep -i permission)
		if [ -n "$MKDIRMSG" ]
		then
		    if [ "$(uname)" = "AIX" ]
		    then
			MKDIRMSG=$(print $MKDIRMSG|grep "The file access permissions do not allow the specified action")
		    else
			MKDIRMSG=$(print $MKDIRMSG|grep "Permission denied")
		    fi
			#$NLS-E$ "You don't have permission to create $CANDLEHOME. \nPlease use another directory for IBM Tivoli Monitoring directory."
    		    [ -n "$MKDIRMSG" ] && print "`nls_replace 'KCI1073I' $CANDLEHOME` \n `nls_replace 'KCI1074I'`"
		    created=false
		    repeat=false
		else
                    chmod a-s $CANDLEHOME > /dev/null 2>&1
                    created=true
		fi
             else
			   #$NLS-E$ "invalid response, please enter \"y\" or \"n\""
               Err_report warn "`nls_replace 'KCI1075I' \\"1\\" \\"2\\"`"
	     created=false
	     repeat=true
             fi
             ;;
        esac
      fi
    fi
    if [ "$repeat" = "false" ]
    then
	   #$NLS-E$ "please create \"$CANDLEHOME\" and rerun"
       [ -d "$CANDLEHOME" ] || Err_report fail "`nls_replace 'KCI1076E' \\"$CANDLEHOME\\"`"
	   #$NLS-E$ "created ITM home directory \"$CANDLEHOME\""
       Err_report info "`nls_replace 'KCI1077I' \\"$CANDLEHOME\\"`"
    fi
  else # The directory $CANDLEHOME exists
    if [ "$created"  != "true" ]
    then
      [ -n "$silentMode" ] && break
	  
	  repeatAsk=1
	  while [ $repeatAsk -eq 1 ]; do
	    #$NLS-E$ "\nITM home directory \"$CANDLEHOME\" already exists."
        print 
	    nls_replace 'KCI1078I' \"$CANDLEHOME\"
	    #$NLS-E$ "OK to use it [ y or n; \"y\" is default ]?  \c"
        print "`nls_replace 'KCI1079I'` [ 1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`; \"1\" `nls_replace 'KCI0199I'` ]?  \c"
        read
        [ -z "$REPLY" ] && REPLY=1
        case $REPLY in
          n*|N*|2) CANDLEHOME=  # Unset CANDLEHOME, and try again.
                 repeatAsk=0
                 ;;
              y*|Y*|1)  repeatAsk=0
                 break 2 # It's safe to use.
                 #break loop 1. [confirming] and loop 2. [entering path]
                 ;;
        esac
	  done
    else
      break  # It's safe to use.
    fi
  fi
done

[[ $CANDLEHOME = @(.|./*) ]] && CANDLEHOME=$(pwd)/$CANDLEHOME  # Provide a full path if "." was entered.
#$NLS-E$ "cannot write to \"$CANDLEHOME\""
[ -w "$CANDLEHOME" ] || Err_report fail "`nls_replace 'KCI1080E' \\"$CANDLEHOME\\"`"

if [ $(print "$CANDLEHOME" | wc -w) -ne 1 ]
then
  #$NLS-E$ "Blanks are not permitted in CANDLEHOME"
  Err_report fail "`nls_replace 'KCI0682E' CANDLEHOME`"
fi

# @10 - ensure that CANDLEHOME is a full pathname.

typeset curDir=$(pwd)
cd $CANDLEHOME > /dev/null 2>&1
#$NLS-E$ "could not determine the full pathname of \"$CANDLEHOME\""
[ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1081E' \\"$CANDLEHOME\\"`"
CANDLEHOME=$(pwd)
cd $curDir > /dev/null 2>&1  # Since this is a retrofit, we'll return to where we started.
#$NLS-E$ "could not change to directory \"$curDir\""
[ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1082E' \\"$curDir\\"`"

# End @10

# See if CANDLEHOME is a soft link.  If so, use the real directory name.

if [ -L $CANDLEHOME ]
then
  set -A linkRec $(ls -l $CANDLEHOME)
  numEle=${#linkRec[*]}
  #$NLS-E$ "could not resolve the link for \"$CANDLEHOME\""
  [ numEle -gt 0 ] || Err_report fail "`nls_replace 'KCI1083E' \\"$CANDLEHOME\\"`"
  CANDLEHOME=${linkRec[numEle-1]}
fi

export CANDLEHOME 
export cArch
export newRegPath

# check if secure group passed to install.sh is valid
[[ -n "$SECURE_GROUP" ]] && validateSecureGroup

#If pdks is not installed in correct version, log message to log file (for silent installation)
mkdir -p $CANDLEHOME/logs > /dev/null 2>&1
logFile=$CANDLEHOME/logs/candle_installation.log
[ number19 -gt number20 ] && print  "`nls_replace 'KCI1318I'`" >>  $logFile 2>&1

#This is in case we overinstall a JRE.
chmod -R 755 $CANDLEHOME/JRE > /dev/null 2>&1

PROD_PATH=${PROD_PATH:-$(pwd)}  # e.g. /buildi/99r1f2/image
[[ $PROD_PATH = @(.|./*) ]] && PROD_PATH=$(pwd)/$PROD_PATH  # Provide a full path if "." was entered.
#$NLS-E$ "\"$PROD_PATH\" is not a directory"
[ -d $PROD_PATH ] || Err_report fail "`nls_replace 'KCI1084E' \\"$PROD_PATH\\"`"
#$NLS-E$ "cannot read \"$PROD_PATH\""
[ -r $PROD_PATH ] || Err_report fail "`nls_replace 'KCI1085E' \\"$PROD_PATH\\"`"

# Set the package location.  The packages could be in a local directory (if downloaded from the Internet) or
# in the "unix" subdirectory of a CD file system.

PKGLOC=""

for pkg in $instPkgs
do
  if [ -f $PROD_PATH/$pkg ]  # Packages found in a local directory.
  then
    PKGLOC=$PROD_PATH
    break
  fi
done

# If no packages were found in a local directory, then they must be on a CD where they will be stored
# in the "unix" subdirectory.
if [ -z "$PKGLOC" ]
then
  tmp=$PROD_PATH/unix
  #$NLS-E$ "\"$tmp\" is not a directory"
  [ -d $tmp ] || Err_report fail "`nls_replace 'KCI1086E' \\"$tmp\\"`"
  #$NLS-E$ "cannot read \"$tmp\""
  [ -r $tmp ] || Err_report fail "`nls_replace 'KCI1087E' \\"$tmp\\"`"
  for pkgs in $instPkgs
  do
    for pkg in $(ls $tmp/$pkgs 2>/dev/null)
    do
      if [ -f $pkg ]  # Packages found on the CD.
      then
        PKGLOC=$tmp
        break
      fi
    done
  done
fi

#$NLS-E$ "could not find any product packages under directory \"$PROD_PATH\""
[ -n "$PKGLOC" ] || Err_report fail "`nls_replace 'KCI1088E' \\"$PROD_PATH\\"`"

# @10 - removed the following restriction.
#[ "$CANDLEHOME" != "$PKGLOC" ] || Err_report fail "product path should not be set to CANDLEHOME - \"$CANDLEHOME\""

# START - determine the variables needed by CandleRemote.

# Determine the location of the base installation script(s), *install.sh.  This will be either the $PKGLOC
# directory or the parent directory of $PKGLOC.

prodMode=""
tarDir=""

if [[ -f $PKGLOC/install.sh || -f $PKGLOC/cinstall.sh || -f $PKGLOC/ninstall.sh ]]
then
  tarDir=$PKGLOC  # Execute the tar command at the location of *install.sh.
  prodMode="eDelivery"  # The quotes are just for readability.
else
  parDir=$(dirname $PKGLOC)
  if [[ -f $parDir/install.sh || -f $parDir/cinstall.sh || -f $parDir/ninstall.sh ]]
  then
    tarDir=$parDir
    prodMode="newCD"  # Just a guess for now.  See following code.
  fi
fi

#$NLS-E$ "could not locate the base installation script"
[ -n "$prodMode" ] || Err_report fail "`nls_replace 'KCI1089E'`"

# Determine where the .ver files are located.  For "eDelivery," they are always inside the .jar files.

if [ "$prodMode" = "newCD" ]
then
  [ -d $PKGLOC/registry ] && prodMode="oldCD"
fi

# END - determine the variables needed by CandleRemote.

# Backup the registry so we can refer to previous version numbers during the
# install for possible data migration purposes.
# Non-compliant installs like Roma(CT98) will just get a token file in the 
# backup area.
RegPath=$CANDLEHOME/registry
RegBackup=$CANDLEHOME/old_registry
typeset LDAPVer=""
[ -d $RegBackup ] || mkdir $RegBackup

# Check for a pre-CT99 roma.  /roma will exist and ideally /registry
# will not.  However, someone may have aborted an install, so we look
# in the registry for some common Roma ver files, and if there are none
# then we have a pre-CT99 roma for sure, and we create a token file.

# If we do find the common Roma ver files, we clean out the backup registry
# in preparation for copying the ver files.  This also deletes any token file. 
[ "$thisOS" = "AIX" ] && os="aix"

if [ "$thisOS" = "HP-UX" ]
then
os=hp
[ "$(uname -m)" = "ia64" ] && os=hpi
fi

[ "$thisOS" = "SunOS" ] && os="sol"
[ "$thisOS" = "Linux" ] && os="l"

RomaPath=$(find $CANDLEHOME/* -name ldap -print | grep "roma/$os" | head -1) #IZ75211
if [ -n "$RomaPath" ]
then
  xx=$(ls $RegPath/bx${os}*.ver 2>/dev/null)
  xx=${xx}$(ls $RegPath/xm${os}*.ver 2>/dev/null)
  xx=${xx}$(ls $RegPath/xc${os}*.ver 2>/dev/null)
  xx=${xx}$(ls $RegPath/xs${os}*.ver 2>/dev/null)
  if [ -z "$xx" ] 
  then
    print $RomaPath > $RegBackup/Pre_CT99_Roma
    chmod 755 $RegBackup/Pre_CT99_Roma > /dev/null 2>&1
    LDAPVer="2.10"
  else	
    # clean out the backup registry
    rm -f $RegBackup/* > /dev/null 2>&1

    # Determine the previous version of LDAP installed (if any)
    ver=$(grep "ver =" $RegPath/xs${os}*.ver 2>/dev/null|head -1|cut -d" " -f3)
    [ -n "$ver" ] && LDAPVer=$(print $ver|cut -c 1).$(print $ver|cut -c 2-3)
  fi
fi

[ -d $RegPath ] && cp $RegPath/*.ver $RegBackup > /dev/null 2>&1

# Create the CANDLEHOME directories.

cd $CANDLEHOME > /dev/null 2>&1
#$NLS-E$ "could not cd to \"$CANDLEHOME\" directory"
[ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1090E' \\"$CANDLEHOME\\"`"

for dir in $reqDirs
do
  [ -d $dir ] || mkdir $dir > /dev/null 2>&1
  #$NLS-E$ "could not create directory \"$CANDLEHOME/$dir\""
  [ -d $dir ] || Err_report fail "`nls_replace 'KCI1091E' \\"$CANDLEHOME/$dir\\"`"
  [ -w $dir ] || chmod 755 $dir > /dev/null 2>&1
  #$NLS-E$ "could not add write access to directory \"$CANDLEHOME/$dir\""
  [ -w $dir ] || Err_report fail "`nls_replace 'KCI1092E' \\"$CANDLEHOME/$dir\\"`"
done

# The name of the docs directory has been changed to doc for consistency with NT.

if [ -d $CANDLEHOME/docs ] 
then
# Skip the error handling since an empty docs dir will result in an error.
  mv $CANDLEHOME/docs/* $CANDLEHOME/doc/ > /dev/null 2>&1
  rmdir $CANDLEHOME/docs > /dev/null 2>&1
fi

export CANDLETMP=$CANDLEHOME/tmp
curRegPath=$CANDLEHOME/registry
refVFile=$CANDLEHOME/registry/citnt.ver

# LL - start token file stuff.

baseTok=$CANDLETMP/candle_install_

thisOS=$(uname -a | cut -d" " -f1)  # thisOS -> AIX
#

# tar file option for unix and root use oxf. defect 12436
# set flag to do chown and chmod for Linux.  defect 83100
tarOpt=xf
chownRoot=n
if [[ "$thisUser" = "root" ]]
then
  if [[ "$thisOS" != "Linux" ]]
  then
    tarOpt=oxf
  else
    chownRoot=y
  fi
fi

if [[ "$thisOS" = NONSTOP_KERNEL ]]
then
  tarOpt=oxf
fi

# 17037 if we are root, we can cleanup unused modules and libs
[[ "$thisUser" = "root" &&  "$thisOS" = "AIX" ]] && slibclean 2>/dev/null

# Create the token file.

thisMach=$(uname -a | cut -d" " -f2)  # thisMach -> vulcan
thisTok=$baseTok${thisMach}_$$
touch $thisTok > /dev/null 2>&1
#$NLS-E$ "could not create token file"
[ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1093E'`"

# Build the description record, and add it to the token file.  This record will look like: 
# "AIX vulcan billy 17440 Friday Jun 08 14:21 2001 |159|".  The number between pipes (|)
# is the day of the year the file was created.

thisRec="$thisOS $thisMach $thisUser $$ $(date +"%A %B %d %H:%M %Y |%j|")"
print "$thisRec" > $thisTok

# Delete old token files, and warn if there is another install running in this <candlehome>.

procTokFiles 

# LL - end token file stuff.


#block upgrade from releases older than 6.1
blockOmegamonUpgrade

#  Skip this step during remote deploys to allow for the case in which prereqs are already installed and are not part of the deploy package
if [[ -z "$DEPLOY_MODE" ]] 
then
  #Defect 78393
  #Determine if the machine's architecture is deprecated in new version before unpacking installer
  dynShl=$CANDLEHOME/bin/dynarch.shl
  #check if scirpt exist and it is executeble (This means that ITM is installed on this machine/location)
  if [ -x $dynShl ] && [ -f $CANDLEHOME/registry/archdsc.tbl ]
  then 

  # To determine the architecture, use payloads from the installation media and
  # not resources already installed in CANDLEHOME.
	oldPROGNAME=$PROGNAME
    checkPlatformBeforeUpgrade 
    PROGNAME=$oldPROGNAME
  fi
fi

su_value=$(locateCMD su);                                               #IZ94626

checkCandleManage # Look for running CandleManage.

checkProcesses  # LL - look for running agents/CMS.

# 17037 if we are root, we can cleanup unused modules and libs
[[ "$thisUser" = "root" &&  "$thisOS" = "AIX" ]] && slibclean 2>/dev/null

cpRMFile  # Copy/rename the readme file.

hasCCC=""
hasRoma=""
hasCMS=""
#shoPrequis
#$NLS-E$ "searching for product families; please wait"
Err_report info "`nls_replace 'KCI1094I'`"

checkDiskSpace $CANDLEHOME 1000

findFamilies  # Determine what product families are available for installation.

# RQ (Candle Application Services Pac) is not allowed to install over Roma
# but is allowed to install over RQ (which has Roma components)
if [ -n "$hasCASP" ]
then
  #$NLS-E$ "this product does not install over eBP"
  [[ -z "$hadCASP" && -n "$hadRoma" ]] && Err_report fail "`nls_replace 'KCI1095E' eBP`"
fi

# If packages are available for more than one product family, ask the user to select a family.

instFam=""
#$NLS-E$ "Exit install."
exInst="`nls_replace 'KCI1096I'`"

if [[ -n "$hasCCC" && -n "$hasRoma" ]]
then
  #$NLS-E$ "Install Candle Commmand Center products."
  cccP="`nls_replace 'KCI1097I'`"
  #$NLS-E$ "Install Roma products."
  romaP="`nls_replace 'KCI1098I'`"

  #$NLS-E$ "\nSelect one of the following:  \n"
  echo
  nls_replace 'KCI1099I'
  echo
  #$NLS-E$ "Please enter a valid number:  "
  PS3="`nls_replace 'KCI1100I'`"
  select num in "$cccP" "$romaP" "$exInst"
  do
    case $num in
      $cccP)
        instFam="-ccc"
        break ;;
      $romaP)
        instFam="-roma"
        break ;;
      $exInst)
        cleanUp 0 ;;
    esac
  done
  echo
fi

# Determine what family to install if packages exist for only one family.

if [ -z "$instFam" ]
then
  if [ -n "$hasCCC" ]
  then 
    instFam="-ccc"
  elif [ -n "$hasRoma" ]
  then
    instFam="-roma"
  fi
fi

# IBM/Tivoli or Candle presentation (use Candle whenever a CMS is in the packaging)
tivoli="-tivoli"
# EJF (#6333) remove this line to allow tivoli to remain set
#[ -n "$hasCMS" ] && tivoli=""

# Ensure that product packages exist.  The check against $PKGLOC is needed since findFamilies() excludes
# ax, ci and jr packages.

#$NLS-E$ "could not find any product packages under directory \"$PROD_PATH\""
[[ -n "$instFam" && -n "$PKGLOC" ]] || Err_report fail "`nls_replace 'KCI1101E' \\"$PROD_PATH\\"`"

# See if any fixpacks exist in $PKGLOC.

hasFixes=false
lookForFixPacks  # If fixpacks are available, hasFixes=true.
#print ... hasFixes = $hasFixes

# See if there are any readme's available for viewing.

hasReadmes=false
ls $CANDLEHOME/doc/readme*.txt > /dev/null 2>&1
[ $? -eq 0 ] && hasReadmes=true

# For eBP over-installs, warn them about shutting down running LDAPs
[[ "$instFam" = "-roma" && -n "$LDAPVer" && -n "$hasLDAP" ]] && BspWarn

# All menu items.

#$NLS-E$ "Install products or maintenance via GUI."
gui="`nls_replace 'KCI1102I'`"
#$NLS-E$ "Install products to the local host."
cmdLn="`nls_replace 'KCI1103I'`"
#$NLS-E$ "Install maintenance via command line."
cmdLnM="`nls_replace 'KCI1104I'`"
#$NLS-E$ "Create remote packages via GUI."
guiRP="`nls_replace 'KCI1105I'`"
#$NLS-E$ "Create remote packages via command line."
cmdLnRP="`nls_replace 'KCI1106I'`"
#$NLS-E$ "View readme files."
vRM="`nls_replace 'KCI1107I'`"
#$NLS-E$ "Install products to depot for remote deployment (requires TEMS)."
cmdLnDepot="`nls_replace 'KCI1108I'`"
#$NLS-E$ "Install products to depot via GUI."
guiDepot="`nls_replace 'KCI1109I'`"
#$NLS-E$ "Install product support."
supportCLI="`nls_replace 'KCI1201I'`"

# Build an accurate list of menu items.

if [ "$hasFixes" = "false" ]
then
  #$NLS-E$ "Install products via GUI."
  gui="`nls_replace 'KCI1110I'`"
fi

# EJF (#12006) remove GUI install option
#mainMenu[0]="$gui"
mainMenu[0]="$cmdLn"
mainMenu2[0]="Install products to the local host."
[ "$hasFixes" = "true" ] && mainMenu[${#mainMenu[*]}]="$cmdLnM"
[ "$hasFixes" = "true" ] && mainMenu2[${#mainMenu2[*]}]="Install maintenance via command line."
# Uncomment the next 2 lines to activate GUI remote package creation, feature 14459
#mainMenu[${#mainMenu[*]}]="$guiRP"
#mainMenu2[${#mainMenu2[*]}]="Create remote packages via GUI."  
# Uncomment the next 2 lines to activate command line remote package creation, feature 14459
#mainMenu[${#mainMenu[*]}]="$cmdLnRP"  
#mainMenu2[${#mainMenu2[*]}]="Create remote packages via command line."  

# Uncomment the next 2 lines to activate GUI depot option
# mainMenu[${#mainMenu[*]}]="$guiDepot"  
# mainMenu2[${#mainMenu2[*]}]="Install products to depot via GUI."  

#
# d15022 abstract      DEV: Remove depot install choice from Exchange Unix installer
# d35743   (modifies this code to add hlv410 and vlv410) 
#
# depot install is not valid for the following cds from the
# install.sh shell script : 
#
# m3v410     IBM Tivoli OMEGAMON Data Files for z/OS
# q5v610     ITM for Microsoft Cluster Servers
# 3zv610     ITM for Windows Active Directory (ADO)
# vwv525     ITM for NetView on z/OS 5.2.5
# hlv410     OMEGAMON z/OS Management Console  
# vlv410     OMEGAMON XE for z/VM and Linux
#
#
# use the 'tacmd addBundles' CLI to add components from these 
# cds to a depot
if [[ "$CDNAME" != @(m3v4*|q5v*|3zv*|vwv*|hlv*|vlv*) ]]; then
    if [[ "$(uname -s)" != "NONSTOP_KERNEL" ]]
    then
    	mainMenu[${#mainMenu[*]}]="$cmdLnDepot"  
		mainMenu2[${#mainMenu2[*]}]="Install products to depot for remote deployment (requires TEMS)."  
    fi
fi
[ "$hasReadmes" = "true" ] && mainMenu[${#mainMenu[*]}]="$vRM"
[ "$hasReadmes" = "true" ] && mainMenu2[${#mainMenu2[*]}]="View readme files."
mainMenu[${#mainMenu[*]}]="$supportCLI"
mainMenu2[${#mainMenu2[*]}]="Install product support"
mainMenu[${#mainMenu[*]}]="$exInst"  
mainMenu2[${#mainMenu2[*]}]="Exit install."  
numItems=${#mainMenu[*]}  # The number of elements in the array.

# Generate a "select" menu, and ask the user to select an install mode.  It's too early in the script 
# to actually invoke the Java installation routines.  Howevever, displaying this menu later in the script 
# would make the install awkward for the user.

while [ anything ]
do
  guiRep=no cmdLnRep=no cmdLnMRep=no guiRPRep=no cmdLnRPRep=no vRMRep=no cmdLnDepotRep=no guiDepotRep=no supportCLIRep=no

  # Force the user to make a selection, unless we're in silent mode.
  if [ -n "$silentMode" ]
  then
    [ "$CLI_OPTION" = "installProduct" ] && cmdLnRep=yes
    [ "$CLI_OPTION" = "installProductToDepot" ] && cmdLnDepotRep=yes
	#$NLS-E$ "installing products via silent mode"
    Err_report info "`nls_replace 'KCI1111I'`"
  else
    i=1 j=0
	#$NLS-E$ "\nSelect one of the following:  \n"
    echo
	nls_replace 'KCI1112I'
	echo
    while [ j -lt numItems ]
    do
      print "$i) ${mainMenu[j]}"
      i=i+1
      j=j+1
    done
  
    while [ anything ]
    do
	  #$NLS-E$ "\nPlease enter a valid number:  \c"
      print "\n`nls_replace 'KCI1113I'`  \c"
      read 
      [[ $REPLY = +([0-9]) ]] || continue  # Make sure it's a number.
      [[ "$REPLY" -gt 0 && "$REPLY" -le numItems ]] && break  # It's a valid number. 
    done
    idx=$REPLY 

    # Determine what the user wants to do.
  
# EJF (#12006) remove GUI install option
#    [ idx -eq 1 ] && guiRep=yes  # The first item is always GUI mode.
    [ idx -eq numItems ] && cleanUp 4  # The last item is always exit.
  
# EJF (#12006) remove GUI install option (changed "idx -gt 1" to "idx -gt 0")
    if [ idx -gt 0 -a idx -lt numItems ]  # Skip the first and last items since they're known.
    then
      idx=idx-1  # Set the real array index.
      sel=${mainMenu2[$idx]}
#      print ... sel = $sel
      foundSth=$(print "$sel" | grep "product" | grep -v "depot" 2> /dev/null)  # Install products via command line.
      [ -n "$foundSth" ] && cmdLnRep=yes
      foundSth=$(print "$sel" | grep "maintenance" 2> /dev/null) # Install maintenance via command line.
      [ -n "$foundSth" ] && cmdLnMRep=yes
      foundSth=$(print "$sel" | grep "remote" 2> /dev/null | grep "GUI" 2> /dev/null) # Create remote packages via GUI.
      [ -n "$foundSth" ] && guiRPRep=yes
      foundSth=$(print "$sel" | grep "remote" 2> /dev/null | grep "command" 2> /dev/null) # Create remote packages via command line.
      [ -n "$foundSth" ] && cmdLnRPRep=yes
      foundSth=$(print "$sel" | grep "readme" 2> /dev/null) # View readme files.
      [ -n "$foundSth" ] && vRMRep=yes
      foundSth=$(print "$sel" | grep "depot" | grep "GUI" 2> /dev/null) # Install products via GUI to agent depot
      [ -n "$foundSth" ] && guiDepotRep=yes
      foundSth=$(print "$sel" | grep "depot" 2> /dev/null) # Install products via command line to agent depot
      [ -n "$foundSth" ] && cmdLnDepotRep=yes
      foundSth=$(print "$sel" | grep "support" 2> /dev/null) # Install product support
      [ -n "$foundSth" ] && supportCLIRep=yes
    fi
  fi
  
  if [ "$vRMRep" = "yes" ]
  then
    typeset rmIdx=""
    typeset -i maxVal=0
    set -A rmDescs
        
    execRMDialog  # Execute the readme dialog.
        
    # Convert the description selected in the menu into a file name and then display the readme.

    if [[ -n "$rmIdx" && "$rmIdx" -lt maxVal ]]  # It must be less than maxVal since maxVal is "exit."
    then  
      desc="${rmDescs[$rmIdx]}"  # Get the description.
      desc=$(print $desc 2> /dev/null | sed "s/ /_/g")  # Add the underscores.
      desc="readme_$desc.txt"  # Complete the file name.
      if [ -f $CANDLEHOME/doc/$desc ]
      then
        more $CANDLEHOME/doc/$desc
      else
	    #$NLS-E$ "could not find readme \"$CANDLEHOME/doc/$desc\""
        Err_report warn "`nls_replace 'KCI1114I' \\"$CANDLEHOME/doc/$desc\\"`"
      fi
    fi
  else
    break
  fi
done
echo

# EJF - per defect 9191, if adding to depot ensure that a TEMS is installed
if [[ "$cmdLnDepotRep" = "yes" || "$guiDepotRep" = "yes" ]] && [[ ! -f $CANDLEHOME/config/.ConfigData/kmsenv ]]
then 
   #$NLS-E$ "No Tivoli Enterprise Monitoring Server (TEMS) is installed under $CANDLEHOME"
   Err_report fail "`nls_replace 'KCI1115E' $CANDLEHOME`"
fi

# Determine where the .ver files are located for new packages.  If there is a "registry" subdirectory in 
# $PKGLOC, then look for the files in the $PKGLOC/registry directory.  .ver files in this directory can
# be read directly.  If there is no "registry" directory under $PKGLOC, the .ver files are contained 
# within the product packages, and these files must be extracted from the packages and placed in $CANDLETMP. 
# We cannot extract to $PKGLOC since this directory may be on a read-only CD.

hasRegDir=no

if [ -d $PKGLOC/registry ]
then
  newRegPath=$PKGLOC/registry
  hasRegDir=yes
else
  newRegPath=$CANDLETMP
  hasRegDir=no
fi

# See if there is a installer (i.e. $ciTar) package in $PKGLOC.  If so, then we need to version and possibly
# update the current installer (if we're installing into an existing CANDLEHOME).

instCI=no
if [ -f $PKGLOC/$ciTar ]
then
  #$NLS-E$ Initializing ...
  nls_replace 'KCI1116I'
  #$NLS-E$ "processing base CI package"
  Err_report info "`nls_replace 'KCI1117I'`"
  
  #$NLS-E$ "cannot read package \"$PKGLOC/$ciTar\""
  [ -r $PKGLOC/$ciTar ] || Err_report fail "`nls_replace 'KCI1118E' \\"$PKGLOC/$ciTar\\"`"
  
  if [ "$hasRegDir" = "no" ] 
  then
    unloadTmpFile $ciVFile $PKGLOC/$ciTar  # Extract the .ver file.
  fi
  
  #$NLS-E$ "cannot read file \"$newRegPath/$ciVFile\""
  [ -r "$newRegPath/$ciVFile" ] || Err_report fail "`nls_replace 'KCI1119E' \\"$newRegPath/$ciVFile\\"`"
  
  # See if our own CI classes are installed.

  if [ -r "$curRegPath/$ciVFile" ]  # Our classes are installed.
  then
    checkLevel $ciVFile  # Check if the "to be installed" CI classes are newer than the ones in CANDLEHOME.
    [ "$levelStrg" = "Same" ] && instCI=Same  # levelStrg is set by function checkLevel().
    [ "$levelStrg" = "Newer" ] && instCI=yes  # levelStrg is set by function checkLevel().
  else  # Our classes are not installed.
    instCI=yes
  fi
fi

# Check if we are trying to install to depot and version of installer on CD is newer then the version in CandleHome, 
# in that case we exit since we do not want to CI upgraded in CH when no components are actually installed just added to depot
if [[ "$cmdLnDepotRep" = "yes" || "$guiDepotRep" = "yes" ]] && [[ "$instCI" = "yes" ]]
then 
  echo
  #$NLS-E$ Media currently being used is unable to install products to depot, use "tacmd" command to populate the remote deploy depot. 
  # Refer to documentation for more details.
  print "`nls_replace 'KCI1177E'`"
  cleanUp 2
fi

   
# Install the installer.

#Defect 81121- moved this setting to the top of Main to make sure we have saved the LANG
#export saveLang=$LANG

savePath=$PATH
  
########################## defect 77379  
if [ "$instCI" = "Same" ]
then
  aFile=tmp/archdsc.tbl
  tar -$tarOpt $PKGLOC/cienv.tar $aFile > /dev/null 2>&1
fi

if [[ "$instCI" = "no" || "$instCI" = "Same" ]]
then
  # Copy the jr<arch>.ver files from the media to the tmp directory if they exist.
  # Otherwise,  assume they were extracted from in cienv.tar.

  jreOS=${cArch%%[0-9]*} #aix, li, sol, hp, etc
  [ -d tmp ] || mkdir tmp > /dev/null 2>&1
  for file in `ls $PKGLOC/jr${jreOS}*.tar $PKGLOC/jr${jreOS}*.ctp 2> /dev/null`
  do
    pkgName=$(basename $file)
    pkgName=${pkgName%.*}
    if [ -f $PKGLOC/$pkgName.ver ]
    then
      cp $PKGLOC/$pkgName.ver tmp/$pkgName.ver > /dev/null 2>&1
    else
      tar -$tarOpt $PKGLOC/cienv.tar tmp/$pkgName.ver > /dev/null 2>&1
    fi
  done
fi
########################## defect 77379  

kbytes=0
if [ "$instCI" = "yes" ]
then
  # See if there is enough room to install.

  export LANG=C
  kbytes=$(grep "^size[A-Z,a-z]* *=" $newRegPath/$ciVFile | cut -d"=" -f2 2> /dev/null)
  export LANG=$saveLang
  #$NLS-E$ "could not determine the size of the base CI package"
  [ kbytes -gt 0 ] || Err_report fail "`nls_replace 'KCI1120E'`"
  
  if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
  then
      kbAvail=10000000
  else
      checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available. 
  fi
  
  # Process any elements.
  
  eleNames="" i=1 numEle=0
  
  export LANG=C
  numEle=$(grep "^numElements *=" $newRegPath/$ciVFile 2> /dev/null | cut -d"=" -f2)
  export LANG=$saveLang
  if [[ $? -eq 0 && numEle -gt 1 ]]  # The installer is contained in multiple packages.
  then
    #$NLS-E$ "processing element CI packages"
    Err_report info "`nls_replace 'KCI1121I'`"
    while [ i -lt numEle ]
    do
      eleNames="$eleNames cienv$i"  # For CI, element names begin with 1.
      i=i+1
    done
    checkCIElements $eleNames
  fi
  
  #$NLS-E$ "unloading CI package(s)"
  Err_report info "`nls_replace 'KCI1122I'`"
  for pkg in $ciTar $eleNames
  do
    bName=${pkg%.*}  # Remove any extension.
    unTarPkg $PKGLOC/$bName.tar 
  done
    
  # Copy the .ver file for the base package, and add write access.
    
  cp $newRegPath/$ciVFile $curRegPath/ > /dev/null 2>&1
  #$NLS-E$ "could not copy version file \"$ciVFile\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1123E' \\"$ciVFile\\"`"
  chmod 755 $curRegPath/$ciVFile > /dev/null 2>&1
  #$NLS-E$ "could not change access permissions on \"$ciVFile\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1124E' \\"$ciVFile\\"`" 

  #46553 cpInstProg removed
  #cpInstProg $ciTar  # Copy install.sh to CANDLEHOME/bin.
  
  # Copying the mo directory and nls_replace file
  cpMo
  
  #65392 backup existing installer translation directory
  rm -f $CANDLEHOME/tmp/nls_msg.tar > /dev/null 2>&1
  tar cf $CANDLEHOME/tmp/nls_msg.tar -C $CANDLEHOME config/nls/msg bin/nls_replace > /dev/null 2>&1
  #$NLS-E$ "could not create translation backup file"
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1238E'`"
  # Copy the jr<arch>.ver files from the media to the tmp directory if they exist.
  # Otherwise,  assume they were extracted from in cienv.tar.

  [ -d tmp ] || mkdir tmp > /dev/null 2>&1
  jreOS=${cArch%%[0-9]*} #aix, li, sol, hp, etc
  for file in `ls $PKGLOC/jr${jreOS}*.tar $PKGLOC/jr${jreOS}*.ctp 2> /dev/null`
  do
    pkgName=$(basename $file)
    pkgName=${pkgName%.*}
    if [ -f $PKGLOC/$pkgName.ver ]
    then
      cp $PKGLOC/$pkgName.ver tmp/$pkgName.ver > /dev/null 2>&1
    fi
  done
  
fi

# Determine if there is an installer (ci) fixpack that needs to be applied.

typeset allVFiles=""

ciFixPacks=$(ls $PKGLOC/ciux*.t* 2> /dev/null)
if [ -n "$ciFixPacks" ]
then
  #$NLS-E$ "processing CI fixpacks"
  Err_report info "`nls_replace 'KCI1125I'`"
  
  for fp in $ciFixPacks
  do
    #$NLS-E$ "cannot read package \"$fp\""
    [ -r $fp ] || Err_report fail "`nls_replace 'KCI1126E' \\"$fp\\"`"
      
    # Determine the name of the .ver file.  If the fixpack name is ciuxdddd.t00 where d is a digit,
    # the .ver file will be named ciuxdddd.ver.  If the fixpack name is ciuxdddd.t01, ciuxdddd.t02,
    # etc, the .ver file will be named ciuxddddt01.ver, ciuxddddt02.ver, etc.
    
    vFile=${fp##*/}   # Remove the pathname.
    ext=${vFile##*.}  # Get the extension; ext -> t00, t01, t02, ...
    if [ "$ext" = "t00" ]
    then
      vFile=${vFile%.*}.ver      # vFile -> ciux6110.ver
    else
      vFile=${vFile%.*}$ext.ver  # vFile -> ciux6110t01.ver
    fi
    
    if [ "$hasRegDir" = "no" ]  # Unload the ciux*.ver files if there is no registry directory.
    then
      unloadTmpFile $vFile $fp  # Extract the .ver file.
    fi
	#$NLS-E$ "cannot read file \"$newRegPath/$vFile\""
    [ -r "$newRegPath/$vFile" ] || Err_report fail "`nls_replace 'KCI1127E' \\"$newRegPath/$vFile\\"`"
    
    [ "$ext" = "t00" ] && allVFiles="$allVFiles $vFile"  # Consider only the base package.
  done

  # Find the name of the .ver file for the latest ci fixpack.
  
  if [ -n "$allVFiles" ]
  then
    findLatest $allVFiles  # findLatest() will place the name of the .ver file in variable lateVFile. 
    ciLateVFile=$lateVFile  # Save the name for future processing.
    checkLevel $ciVFile $ciLateVFile  # See if the fixpack needs to be installed. 
    [ "$levelStrg" = "Newer" ] && instCIFix=yes
  fi 
fi

# Install a ci fixpack.

kbytes=0
if [ "$instCIFix" = "yes" ]
then
  # See if there is enough room.
  
  export LANG=C
  kbytes=$(grep "^size[A-Z]* *=" $newRegPath/$ciLateVFile | cut -d"=" -f2)
  export LANG=$saveLang
  #$NLS-E$ "could not determine the size of the CI fixpack"
  [ kbytes -gt 0 ] || Err_report fail "`nls_replace 'KCI1128E'`"

  if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
  then
      kbAvail=10000000
  else
      checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available. 
  fi
  
  
  # Process any elements.
  
  eleNames="" i=1 numEle=0
  typeset baseName=${ciLateVFile%.*}
  
  export LANG=C
  numEle=$(grep "^numElements *=" $newRegPath/$ciLateVFile 2> /dev/null | cut -d"=" -f2)
  export LANG=$saveLang
  if [[ $? -eq 0 && numEle -gt 1 ]]  # The fixpack is contained in multiple packages.
  then
    #$NLS-E$ "processing element CI fixpacks"
    Err_report info "`nls_replace 'KCI1129I'`"
    while [ i -lt numEle ]
    do
      [ i -le 9 ] && ext=t0$i || ext=t$i
      eleNames="$eleNames $baseName.$ext"  
      i=i+1
    done
    checkCIFixElements $baseName $eleNames
  fi
  
  #$NLS-E$ "installing CI fixpack \"$baseName\""
  Err_report info "`nls_replace 'KCI1130I' \\"$baseName\\"`"
  for pkg in $baseName.t00 $eleNames
  do
    unTarPkg $PKGLOC/$pkg 
  done
    
  # Preserve existing fixID records.  Adding the values to an array will remove the newline
  # character returned from the grep operation.
  
  export LANG=C
  set -A fixIDs $(grep "fixID" $curRegPath/$ciVFile | grep -v $baseName | cut -d"=" -f2 2> /dev/null)
  export LANG=$saveLang
  
  # Copy the base .ver file (while changing its name), and add write access.

  cp $newRegPath/$ciLateVFile $curRegPath/$ciVFile > /dev/null 2>&1
  #$NLS-E$ "could not copy version file \"$ciLateVFile\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1131E' \\"$ciLateVFile\\"`"
  chmod 755 $curRegPath/$ciVFile > /dev/null 2>&1
  #$NLS-E$ "could not change access permissions on \"$ciVFile\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1132E' \\"$ciVFile\\"`"
  
  # Add the fixpack names which have been installed to the end of the .ver file.
  
  print "" >> $curRegPath/$ciVFile 2> /dev/null
  for pkg in ${fixIDs[*]} $baseName
  do
    print "fixID = $pkg" >> $curRegPath/$ciVFile 2> /dev/null
  done
  
  #46553 cpInstProg removed
  #cpInstProg $baseName.t00  # Copy install.sh to CANDLEHOME/bin.
fi

# Copy the kcirunas.cfg file to CANDLEHOME/config if it does not already exist there.

RUNASFILE=kcirunas.cfg
RUNASIMG=""
RUNASCFG="$CANDLEHOME/config/$RUNASFILE"
[[ -z "$RUNASIMG" && -f "$PROGDIR/$RUNASFILE" ]] && RUNASIMG="$PROGDIR/$RUNASFILE"
[[ -z "$RUNASIMG" && -f "$PKGLOC/../$RUNASFILE" ]] && RUNASIMG="$PKGLOC/../$RUNASFILE"
[[ -z "$RUNASIMG" && -f "$PKGLOC/$RUNASFILE" ]] && RUNASIMG="$PKGLOC/$RUNASFILE"
#$NLS-E$ "cannot locate file \"$RUNASFILE\" - please try the install again"
[[ -z "$RUNASIMG" ]] && Err_report fail "`nls_replace 'KCI11072E' \\"$RUNASFILE\\"`"

if [[ ! -f "$RUNASCFG" ]]
then
  cp "$RUNASIMG" "$RUNASCFG"
  #$NLS-E$ "could not copy file \"$RUNASIMG\" to \\"$RUNASCFG\\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI11074E' \\"$RUNASIMG\\" \\"$RUNASCFG\\"`"
fi

# Locate the architecture description table. This table is needed to run dynarch.shl.

archTbl=archdsc.tbl
if [ -r $newRegPath/$archTbl ]  # Use the "new" table if there is one.
then
  archTbl=$newRegPath/$archTbl
else
  archTbl=$curRegPath/$archTbl
fi
#$NLS-E$ "cannot read file \"$archTbl\" - please try the install again"
[ -r $archTbl ] || Err_report fail "`nls_replace 'KCI1133E' \\"$archTbl\\"`"

# Display the license agreement after we've unpacked it
# Command-line modes need to display the license text first

# Which license agreement to display? (default is license.txt)
[ -n "$hasRoma" ]             && license="terms.agr"
[ "$instFam" = "-romaMQSEC" ] && license="mqterms.agr"
[ "$instFam" = "-pqedit" ] && license="terms.agr_MsgEdit"
export LANG=C
licName=$(print $license | tr 'a-z' 'A-Z')
export LANG=$saveLang
# Get user agreement on licensing using LAP tool if its on CD, or Candle license if not
lapDisplayMode="none"

if [ -d "$PROD_PATH/unix/LAP" ]
then
  if [[ "$cmdLnRep" = "yes" || "$cmdLnMRep" = "yes" || "$cmdLnRPRep" = "yes" || "$cmdLnDepotRep" = "yes" ]]
  then
    lapDisplayMode="-text_only"
  else
    lapDisplayMode="-win_style 0"
  fi
else
  # If license file isn't there (maybe deleted), check /misc for a spare copy
  [ -r $license ]  || cp -p $CANDLEHOME/misc/.$license $license 2> /dev/null
  # also check the top level of the cd for the license file
  [ -r $license ]  || cp -p $PROD_PATH/$license $license 2> /dev/null
  
  # Get license agreement unless we're in silent mode
  if [ -z "$silentMode" ]
  then
    if [[ "$cmdLnRep" = "yes" || "$cmdLnMRep" = "yes" || "$cmdLnRPRep" = "yes" || "$cmdLnDepotRep" = "yes" ]]
    then
      while true		# license.txt gets option to postpone view @03
      do
        print " "
		#$NLS-E$ "Do you agree to the terms of the license in file $licName on the CD?"
        print "`nls_replace 'KCI1134I' $licName`"
		#$NLS-E$ "Enter Yes, No, or R to read the agreement now."
        print "1-`nls_replace 'KCI0197I'`, 2-`nls_replace 'KCI0198I'`, `nls_replace 'KCI1135I' 3`"
		#$NLS-E$ "(choosing \"No\" exits the installation process) (Y/N/R): \c"
        print "`nls_replace 'KCI1136I' \\"2\\"` (1/2/3): \c"
        read Response
        case "$Response" in
          n*|N*|2) exit 0
                 ;;
          r*|R*|3) shoLicense
                 ;;
          y*|Y*|1) break
                 ;;
        esac
      done
    fi
  fi
fi

# Determine the machine's architecture.

dynShl=$CANDLEHOME/bin/dynarch.shl
#$NLS-E$ "cannot execute \"$dynShl\""
[ -x $dynShl ] || Err_report fail "`nls_replace 'KCI1137E' \\"$dynShl\\"`"
. $dynShl $archTbl  # dynarch.shl will determine a value for cArch.
#$NLS-E$ "could not determine the machine's architecture"
[ -n "$cArch" ] || Err_report fail "`nls_replace 'KCI1138E'`"
#$NLS-E$ "computed architecture = \"$cArch\""
Err_report info "`nls_replace 'KCI1139I' \\"$cArch\\"`"

#$NLS-E$ "HP 10 is not a supported operating system"
[[ "$cArch" = "hp10" ]] && Err_report fail "`nls_replace 'KCI1140E'`"
#$NLS-E$ "HP 10.20 is not a supported operating system"
[[ "$cArch" = "hp102" ]] && Err_report fail "`nls_replace 'KCI1141E'`"

#
# f14583 abstract      Install support for TEPS on Intel SuSE9
#
# TEPS needs to differentiate between Red and SUSE linux
export LANG=C
if [[ -f /etc/SuSE-release ]]
then
   VERSION=`cat /etc/SuSE-release | grep 'VERSION' | grep -v '^#' | sed  -e 's#[^0-9]##g'`
   SUSE_VERSION="-Dsuse_version=${VERSION}"
fi
export LANG=$saveLang

# Get the names of all JRE packages (products and fixpacks) in the product directory.

typeset jrPkgs=""

# Get a list of the JRE packages we have in the media
export LANG=C
jrPkgs=$(ls $PKGLOC/jr*.tar  $PKGLOC/jr*.ctp $PKGLOC/jr*.z* 2> /dev/null)
export LANG=$saveLang

#always unload JRE packages .ver files to tmp directory
#unless the JRE .ver file is also external to the JRE package
if [ -n "$jrPkgs" ]
then
  #$NLS-E$ "processing JRE packages"
  Err_report info "`nls_replace 'KCI1142I'`"
  for jre in $jrPkgs
  do
  	#
  	# if the jre does not have read permission and 
  	# this is not deploy mode exit
  	if [[ ! -r $jre && -z "$DEPLOY_MODE" ]]
  	then
	    #$NLS-E$ "cannot read package \"$jre\""
  		Err_report fail "`nls_replace 'KCI1143E' \\"$jre\\"`"
  	fi
     
    bName=${jre##*/}  # Remove the pathname.
    
    # Construct the name of the .ver file.
    
    ext=${bName##*.}  # ext -> tar, ctp, z00, z01, z02, ...
    if [[ "$ext" = "tar" || "$ext" = "ctp" || "$ext" = "z00" ]]
    then
      vFile=${bName%.*}.ver      # vFile -> jraix4.ver
    else
      vFile=${bName%.*}$ext.ver  # vFile -> jraix4z01.ver
    fi
    
    # copy jre .ver from installation media if available
    if [[ -f $PKGLOC/$vFile ]]
    then
      cp $PKGLOC/$vFile $CANDLEHOME/tmp/ > /dev/null 2>&1
    else
      unloadTmpFile $vFile $jre  # Extract the .ver file.
    fi
    
	#$NLS-E$ "cannot read file \"$newRegPath/$vFile\""
    [ -r "$newRegPath/$vFile" ] || Err_report fail "`nls_replace 'KCI1144E' \\"$newRegPath/$vFile\\"`"
      
  done
fi

# See if there is a JRE installed.

osn=${cArch%%[0-9]*}  # Get the name (e.g. aix) of the operating system.
#$NLS-E$ "About to check for installed JRE"
Err_report info "`nls_replace 'KCI1145I'`"
curJVF=$(grep -l "^runArch *= *$cArch$" $curRegPath/jr$osn*.ver 2> /dev/null)
if [ -z "$curJVF" ]  # Look for a "runArch = open" record.
then
  export LANG=C
  set -A ta $(grep -l "^runArch *= *open" $curRegPath/jr$osn*.ver 2> /dev/null)
  export LANG=$saveLang
  numEle=${#ta[*]}
  # There may be more than one .ver file with an "open" record, so use the latest .ver file.
  [ numEle -gt 0 ] && curJVF=${ta[numEle-1]}
  
	# 17901.1 - additional check for a 32-bit JRE
  if [ -z "$curJVF" ]  # Look for the 32-bit version if this is 64-bit
  then
    # get architecture bit (3 = 32 or 6 = 64)
    typeset -i len=0
    len=$(print $cArch|wc -c)
    len=$(($len-1))
    bitNum=$(print $cArch|cut -c$len)
    [ -z "$bitNum" ] && bitNum=3

    # if we're 64-bit
    if [ "$bitNum" = 6 ]
    then
	  #$NLS-E$ "64bit JRE not found looking for installed 32bit JRE"
      Err_report info "`nls_replace 'KCI1146I'`"
      newRunArch=$(print $cArch | sed 's/6$/3/g') # replace ending 6 with 3
      export LANG=C
	  set -A ta $(grep -l "^runArch *= *$newRunArch$" $curRegPath/jr$osn+([0-9]).ver 2> /dev/null)
      export LANG=$saveLang
	  numEle=${#ta[*]}
      # There may be more than one .ver file, so use the latest .ver file.
      [ numEle -gt 0 ] && curJVF=${ta[numEle-1]}
	  #$NLS-E$ "current JRE is \"$curJVF\""
      Err_report info "`nls_replace 'KCI1147I' \\"$curJVF\\"`"
    fi
  fi
fi
[ -n "$curJVF" ] && curJVF=${curJVF##*/}  # Remove the pathname.

# See if there is a new JRE to install.

export LANG=C
newJVF=$(grep -l "^runArch *= *$cArch$" $newRegPath/jr$osn+([0-9]).ver 2> /dev/null)  # Exclude *zxx .ver files.
export LANG=$saveLang
if [ -z "$newJVF" ]  # Look for a "runArch = open" record.
then
  export LANG=C
  set -A ta $(grep -l "^runArch *= *open" $newRegPath/jr$osn+([0-9]).ver 2> /dev/null)
  export LANG=$saveLang
  numEle=${#ta[*]}
  # There may be more than one .ver file with an "open" record, so use the latest .ver file.
  [ numEle -gt 0 ] && newJVF=${ta[numEle-1]}

  # 13526.1 - additional check for a 32-bit JRE
  if [ -z "$newJVF" ]  # Look for the 32-bit version if this is 64-bit
  then
    # get architecture bit (3 = 32 or 6 = 64)
    typeset -i len=0
    len=$(print $cArch|wc -c)
    len=$(($len-1))
    bitNum=$(print $cArch|cut -c$len)
    [ -z "$bitNum" ] && bitNum=3

    # if we're 64-bit
    if [ "$bitNum" = 6 ]
    then
      newRunArch=$(print $cArch | sed 's/6$/3/g') # replace ending 6 with 3
      export LANG=C
	  set -A ta $(grep -l "^runArch *= *$newRunArch$" $newRegPath/jr$osn+([0-9]).ver 2> /dev/null)
      export LANG=$saveLang
	  numEle=${#ta[*]}
      # There may be more than one .ver file, so use the latest .ver file.
      [ numEle -gt 0 ] && newJVF=${ta[numEle-1]}
    fi
  fi
fi

[ -n "$newJVF" ] && newJVF=${newJVF##*/}  # Remove the pathname.

# Determine if a JRE needs to be installed.

instJRE=no
#d94520- If we are installing from an application support only dvd, display a more useful error message
#   for this scenario. This would happen if the user is trying to install an application support image that
#   does not contain Java into a directory that does not contain an existing ITM environment. This scenario
#   is not supported.
if [[ "$PLATFORM_SUPPORTED_CHECK" = "NO" ]]
then

   # $NLS-E$ "KCI1292I Installations from application support media must be installed into a directory containing an existing IBM Tivoli Monitoring environment."
   [[ -z "$curJVF" && -z "$newJVF" ]] && Err_report fail "`nls_replace 'KCI1292I'`"
else

   #$NLS-E$ "This is not a supported platform or the installation media does not contain any components supported on this platform."
   [[ -z "$curJVF" && -z "$newJVF" ]] && Err_report fail "`nls_replace 'KCI1148E'`"
fi

#  Skip this step during remote deploys to allow for the case in which prereqs are already installed and are not part of the deploy package
if [[ -z "$DEPLOY_MODE" ]] 
then
  #double check if this is supported platform, by checking jr & gs .ver files.
  if [[ $platformChecked = "NO" ]]; then
    isPlatformSupported $cArch;
    [[ $? -ne 0 ]] && Err_report fail "`nls_replace 'KCI1148E'`"
  fi
fi

[[ -n "$curJVF" && -z "$newJVF" ]] && { instJRE=no; toJVF=$curJVF; }

[[ -z "$curJVF" && -n "$newJVF" ]] && { instJRE=yes; toJVF=$newJVF; }

if [[ -n "$curJVF" && -n "$newJVF" ]]
then
  # See if the JRE in the product directory is newer than the one in CANDLEHOME.
  checkLevel $curJVF $newJVF
  if [ "$levelStrg" = "Newer" ] 
  then
    instJRE=yes
    toJVF=$newJVF
  else
    instJRE=no
    toJVF=$curJVF
  fi
fi

# Install the JRE.

kbytes=0
if [ "$instJRE" = "yes" ]
then
  # See if there is enough room to install.
  
  export LANG=C
  kbytes=$(grep "^size[A-Z,a-z]* *=" $newRegPath/$newJVF 2> /dev/null | cut -d"=" -f2)
  export LANG=$saveLang
  #$NLS-E$ "could not determine the size of the base JRE package"
  [ kbytes -gt 0 ] || Err_report fail "`nls_replace 'KCI1149E'`"

  if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
  then
      kbAvail=10000000
  else
      checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available. 
  fi
  
  
  # Determine the name of the base package.
  
  bName=${newJVF%.*}  # Remove the .ver extension.
  bPkg=$(ls $PKGLOC/$bName.z00 2> /dev/null)  # Look first for the latest .z package
  [ -n "$bPkg" ] || bPkg=$(ls $PKGLOC/$bName.tar $PKGLOC/$bName.ctp 2> /dev/null)
  if [ -n "$bPkg" ] 
  then
    bPkg=${bPkg##*/}  # Remove the pathname
  else
    # 13526.1 - for remote deploy, there may not be a native 64-bit JRE

    # get archtecture bit (3 = 32 or 6 = 64)
    typeset -i len=0
    len=$(print $cArch|wc -c)
    len=$(($len-1))
    bitNum=$(print $cArch|cut -c$len)
    [ -z "$bitNum" ] && bitNum=3

    # if we're 64-bit and can't find a 64-bit JRE, look for 32-bit
    if [ "$bitNum" = 6 ]
    then
      newRunArch=$(print $cArch | sed 's/6$/3/g') # replace ending 6 with 3
      export LANG=C
      set -A ta $(grep -l "^runArch *= *$newRunArch$" $newRegPath/jr$osn+([0-9]).ver 2> /dev/null)
      export LANG=$saveLang
	  numEle=${#ta[*]}
      # There may be more than one .ver file, so use the latest .ver file.
      [ numEle -gt 0 ] && newJVF=${ta[numEle-1]}
      [ -n "$newJVF" ] && newJVF=${newJVF##*/}  # Remove the pathname.
    fi
    bName=${newJVF%.*}  # Remove the .ver extension.
    bPkg=$(ls $PKGLOC/$bName.z00 2> /dev/null)  # Look first for the latest .z package
    [ -n "$bPkg" ] || bPkg=$(ls $PKGLOC/$bName.tar $PKGLOC/$bName.ctp 2> /dev/null)

    if [ -n "$bPkg" ]
    then
      bPkg=${bPkg##*/}  # Remove the pathname
    else
	  #$NLS-E$ " could not find a base JRE package in \"$PKGLOC\""
      Err_report fail "`nls_replace 'KCI1150E' \\"$PKGLOC\\"`"
    fi
  fi
 
  eleNames="" i=1 numEle=0
  if [ "$bPkg" = "$bName.z00" ]  # There may be elements, so process them.
  then
    numEle=$(grep "^numElements *=" $newRegPath/$newJVF 2> /dev/null | cut -d"=" -f2)
    if [[ $? -eq 0 && numEle -gt 1 ]]  # The JRE is contained in multiple packages.
    then
	  #$NLS-E$ "versioning element JRE packages"
      Err_report info "`nls_replace 'KCI1151I'`"
      while [ i -lt numEle ]
      do
        [ i -lt 10 ] && eleNames="$eleNames $bName.z0$i"
        [ i -ge 10 ] && eleNames="$eleNames $bName.z$i"
        i=i+1
      done
      checkJRElements $eleNames
    fi
  fi

  # EJF (#12868) If a JRE is going to be overwritten remove it first
  # (41680) we should remove base JRE only. There might be preReq JRE, we shouldn't remove it.
  # (53460) we should remove the JRE only, which will be overwritten.
  jrName=$(print $curJVF | cut -d . -f1 | cut -c3-)
  if [[ -n "$jrName" && -d $CANDLEHOME/JRE/$jrName ]]
  then
    #$NLS-E$ "removing old JRE"
    Err_report info "`nls_replace 'KCI1152I'`"   
    if [ "$curJVF" = "$newJVF" ]
    then
    	rm -rf $CANDLEHOME/JRE/$jrName > /dev/null 2>&1
    	if [[ $? -eq 0 ]]
    	then
	  	#$NLS-E$ "old JRE has been removed"
      	Err_report info "`nls_replace 'KCI1153I'`"
      	if [[ -n "$curRegPath" && -n "$curJVF" && -f "$curRegPath/$curJVF" ]]
      	then
			#--------------------------------------------------------------#IZ95119
			# In lieu of removing the ver file, delete its content so we   #IZ95119
			# maintain its ownership and permissions when the new version  #IZ95119
			# takes its place later on below.                              #IZ95119
			#--------------------------------------------------------------#IZ95119
			print -n "" > $curRegPath/$curJVF 2>&1                         #IZ95119
			#$NLS-E$ "could not remove old JRE version file \"$curJVF\""
        	[ $? -eq 0 ] || Err_report warn "`nls_replace 'KCI1154I' \\"$curJVF\\"`"
      	fi
    	else
	  	#$NLS-E$ "unable to remove all or part of old JRE"
      	Err_report warn "`nls_replace 'KCI1155I'`"
    	fi
    fi     
  fi 
  
  #$NLS-E$ "unloading JRE package(s)"
  Err_report info "`nls_replace 'KCI1156I'`"
  for pkg in $bPkg $eleNames
  do
    unTarPkg $PKGLOC/$pkg 
    if [ "$(uname -s)" = "NONSTOP_KERNEL" ] 
    then
	while true
	do
  	    repeat=true
  	    if [ -z "$JAVALOC" ]
	    then
		        #$NLS-E$ "\nEnter the java installed location.(Default is /usr/tandem/java)"
				echo
    	        nls_replace 'KCI1157I' /usr/tandem/java
				export LANG=C
    	        read resp    	        
				resp=$(eval print $resp)
    	        export LANG=$saveLang
				[ -z "$resp" ] && resp=/usr/tandem/java	
	    else
		resp="$JAVALOC"
	    fi
	
	    if [[ ! -d "$resp" ]]
	    then
		#$NLS-E$ "\n$resp is not a correct directory."
		echo
		nls_replace 'KCI1158I' $resp
		[ -n "$silentMode" ] && exit	# if this is silent mode exit now
		JAVALOC=""
		continue
	    else
		if [ ! -x $resp/jre/bin/java ] 
		then
		    #$NLS-E$ "\nThe $resp/jre/bin/java is not an executable file."
			echo
		    nls_replace 'KCI1159I' $resp/jre/bin/java
		    [ -n "$silentMode" ] && exit		# silent mode exit here.
		    JAVALOC=""
		    continue
		fi
	    fi

	    JreVersion=1.4.2
	    JreVersion15=1.5
	    jreCmd="$resp/jre/bin/java -version"
	    [ -f /tmp/jrever.txt ] && rm -f /tmp/jrever.txt
	    $jreCmd 2>/tmp/jrever.txt
	    jrever=$(grep -e "$JreVersion" -e "$JreVersion15" /tmp/jrever.txt)
	    [ -f /tmp/jrever.txt ] && rm -f /tmp/jrever.txt
	    if [[ -z $jrever ]] 
	    then
		$resp/jre/bin/java -version
		#$NLS-E$ "\nThis is not a correct jre version. You have to install jre 1.4.2 or 1.5."
		print 
		nls_replace 'KCI1175E'
		exit
	    fi
	    JAVALOC=$resp
	    break
	done
        ln -sf $JAVALOC/jre/bin $CANDLEHOME/JRE/$cArch/bin
        ln -sf $JAVALOC/jre/lib  $CANDLEHOME/JRE/$cArch/lib
    fi
  done
  
  #45988.1
  chmod -R 755 $CANDLEHOME/JRE > /dev/null 2>&1
    
  # Copy the .ver file for the base package, and add write access.

  cp $newRegPath/$newJVF $curRegPath/ > /dev/null 2>&1
  #$NLS-E$ "could not copy version file \"$newJVF\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1161E' \\"$newJVF\\"`"
  chmod 755 $curRegPath/$newJVF > /dev/null 2>&1
  #$NLS-E$ "could not change access permissions on \"$newJVF\""
  [ $? -eq 0 ] || Err_report fail "`nls_replace 'KCI1162E' \\"$newJVF\\"`"
fi

# Copy the tables only if they don't already exist in CANDLEHOME/registry.  Remember, the old and new tables
# will be merged by CandleInstall.

for txt in $tables
do
  if [ ! -r $curRegPath/$txt ]
  then
    if [ -r $newRegPath/$txt ]
    then 
      cp $newRegPath/$txt $curRegPath/ > /dev/null 2>&1
      chmod 755 $curRegPath/$txt > /dev/null 2>&1
    fi
  fi
  
  #$NLS-E$ "cannot read file \"$curRegPath/$txt\""
  [ -r $curRegPath/$txt ] || Err_report fail "`nls_replace 'KCI1163E' \\"$curRegPath/$txt\\"`"
done

  if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
  then
      kbAvail=10000000
  else
      checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available. 
  fi
  

# The JRE is now installed under $CANDLEHOME/JRE.

[ -d "$CANDLEHOME/$oldJRE" ] && mv $CANDLEHOME/$oldJRE $CANDLEHOME/$oldJRE.old > /dev/null 2>&1

# Determine which JRE to use (The JRE name is stored in env var JREEXE.), and set the CLASSPATH. 
# Function setJavaEnv is defined in dynarch.shl.

setJavaEnv $cArch



# For Roma 2.1 overinstalls, we may have to rename some directories
[ "$instFam" = "-roma" ] && Bsp21Rename $RegBackup

# Display the LAP license agreement, if it was packaged
[ "$lapDisplayMode" != "none" ] && run_LAPTool $lapDisplayMode

# now copy required GSkit package to CANDLEHOME/tmp/GSkit directory
  
#Mliao

  # Copy GSkit files depending on the platform type and bitMode
  os=$(print $cArch|cut -c1-2)
  hw=$(print $cArch|cut -c2-3)

  # Extract the bitMode (32/64) of the platform
  typeset -i len=0
  len=$(print $cArch|wc -c)
  len=$(($len-1))
  bitNum=$(print $cArch|cut -c$len)
  [ -z "$bitNum" ] && bitNum=3


  #
  # 12150.1 abstract      SVT: xLinux & zLinux depot install missing GSKit files
  # only copy GSKit if installing products to the local box - not when installing
  # products to the depot
  #
  # also, the GSKit does not exist on the ADO and Exchange CDs - don't copy 
  # when installing these cds

  # if this is Tandem we will not install and support gskit
OSTYPE=$(uname -s)

if [[ "$CDNAME" != @(m3v*|hlv*|3zv*) ]] && [[ "$cmdLnRep" = "yes" || "$guiRep" = "yes" ]] && [[ "$OSTYPE" != "NONSTOP_KERNEL" ]]
then
  setSomeCandleEnvironment
  setGSKtarnames
  # run only if GSKit product directory exists
  if [ -n "$gskTar" ]; then   
    checkGSKtarnames
	  if [ -z "$gskArch" ] ; then
	    cleanUp 0
	    return
	  fi

    exportGskitVariables

    [[ -f "$CANDLEHOME/config/gsKit.config" ]] && localGSKitEnabled="yes"

    cmd=$CANDLEHOME/bin/runGSkit
  
	  if [ -x $cmd ]
	  then
	    if [ "X$silentMode" != X ]
	    then
	      runGSkitParm='-mode silent'
	    elif [ $guiRep = yes ]
	    then
	      runGSkitParm='-mode gui'
	    else
	      runGSkitParm='-mode cmd'
	    fi


      # unset LD_LIBRARY_PATH before installing GSkit
      typeset athisOS=$(uname -s) savedLibraryPath=$LD_LIBRARY_PATH
      if [ "$athisOS" = "HP-UX" ]; then
        unset LD_LIBRARY_PATH
      fi
	
      runCommandPerfect $cmd $runGSkitParm # Run the command.

      if [ "$athisOS" = "HP-UX" ]; then export LD_LIBRARY_PATH=$savedLibraryPath; fi
      
	    fi
	  fi
fi

# Recalculate available space and insure we have enough space for temporary files 
# and logging to start the next phase of the installation process.

kbytes=10000
if [[ "$(uname -s)" = "NONSTOP_KERNEL" ]]	# Tandem df doesn't work as we expect, hard code now.
then
    kbAvail=10000000
else
    checkDiskSpace $CANDLEHOME $kbytes  # install.sh exits if space is not available. 
fi
 
checkSYHDupgrade pre

# Start the installer.
if [[ "$supportCLIRep" = "yes" ]]
then
cmdLnRep="no"
cmdLnDepotRep="no"
startSupport
fi
[ "$guiRep" = "yes" ] && startGUI
[ "$cmdLnRep" = "yes" ] && startCmdLn prod
[ "$cmdLnMRep" = "yes" ] && startCmdLn fix
[ "$guiRPRep" = "yes" ] && startGUIRP
[ "$cmdLnRPRep" = "yes" ] && startCmdLnRP
[ "$cmdLnDepotRep" = "yes" ] && startCmdLnDepot
[ "$guiDepotRep" = "yes" ] && startGUIDepot

# Restore the library path.  It was saved in dynarch.shl.  The restoration must be done after CandleInstall 
# has completed.

if [ -n "$saveLibPath" ]
then
  [[ "$1" = aix+([0-9]) ]] && export LIBPATH="$saveLibPath"
  [[ "$1" = hp+([0-9]) ]] && export SHLIB_PATH="$saveLibPath"
  [[ "$1" = hpi+([0-9]) ]] && export SHLIB_PATH="$saveLibPath"
  [[ "$1" = sol+([0-9]) ]] && export LD_LIBRARY_PATH="$saveLibPath"
  [[ "$1" = tsf+([0-9]) ]] && export LD_LIBRARY_PATH="$saveLibPath"
fi

export LANG="$saveLang"
export PATH="$savePath"

# change owner and group fot Linux if ID is root defect 12436
if [[ "$thisOS" = "Linux" && "$thisUser" = "root" ]]
then
    chown -R root $CANDLEHOME/bin $CANDLEHOME/config $CANDLEHOME/HTML $CANDLEHOME/images $CANDLEHOME/samples $CANDLEHOME/misc $CANDLEHOME/JRE  $CANDLEHOME/classes  $CANDLEHOME/tmp
    chgrp -R root $CANDLEHOME/bin $CANDLEHOME/config $CANDLEHOME/HTML $CANDLEHOME/images $CANDLEHOME/samples $CANDLEHOME/misc $CANDLEHOME/JRE  $CANDLEHOME/classes  $CANDLEHOME/tmp
    [ -d $CANDLEHOME/tables ] && chmod -R 777 $CANDLEHOME/tables
fi

if [[ -z "$SECURE_GROUP" ]];
then
  # Set write permissions for some directories 
  chmod -R 777 $CANDLEHOME/bin 2> /dev/null
  chmod -R 777 $CANDLEHOME/config 2> /dev/null
  chmod -R 777 $CANDLEHOME/logs 2> /dev/null
  chmod -R 777 $CANDLEHOME/tmp 2> /dev/null

  # These products require more write permissions than others

  # defect 18183, the permssions set in SetPerm are being overwritten here  
  chmod -R 777 $CANDLEHOME/*/um/work 2> /dev/null
  chmod -R 777 $CANDLEHOME/*/um/tables 2> /dev/null
  chmod -R 777 $CANDLEHOME/*/um/scripts 2> /dev/null
  chmod -R 777 $CANDLEHOME/*/um/metafiles 2> /dev/null
  chmod -R 777 $CANDLEHOME/*/qi 2> /dev/null

  # EJF (#12046) ensure that the images directory has read access for everyone
  chmod -R 755 $CANDLEHOME/images 2> /dev/null
fi 
  
# set AUTORUN_SECURE variable to skip secureMain execution in UpdateAutoRun.sh
export AUTORUN_SECURE=no

# defect 44550, run UpdateAutoRun.sh only for root user
# $CANDLEHOME/bin/UpdateAutoRun.sh $CANDLEHOME
if [ "${thisRealUser}" = "root" ]; then                                 #IZ94626.1
    $CANDLEHOME/bin/UpdateAutoRun.sh -h $CANDLEHOME
else
    # For non-root user, just display a warning
    #$NLS-E$ "The root user must run bin/UpdateAutoRun.sh from the ITM home to complete the setup"
    Err_report warn "`nls_replace 'KCI1042I' bin/UpdateAutoRun.sh`"
    
    # For non-root on aix system, display warning 
    #$NLS-E$ "The root user must run 'slibclean' utility and restart all components afterwards"
    [[ $os = "ai" ]] && [[ -z "$localGSKitEnabled" ]] && [[ -f "$CANDLEHOME/config/gsKit.config" ]] && Err_report warn "`nls_replace 'KCI1255I'`"
fi

# secure ITM installation
$CANDLEHOME/bin/itmsecure.sh install

checkSYHDupgrade post
cd $CANDLEHOME/bin
restart_proc start

if [[ -n "$syNotStart" || -n "$hdNotStart" ]];
then
  oldSYver=$(print $oldSYver | sed 's/[ ]*$//')
  oldHDver=$(print $oldHDver | sed 's/[ ]*$//')
  if [[ -n "$syNotStart" && ! $oldSYver = "610" ]] || [[ -n "$hdNotStart" && ! $oldHDver = "610" ]];
  then
    nls_replace 'KCI1317I'
    else
    nls_replace 'KCI1251I'
  fi 
fi 

cd - 2> /dev/null > /dev/null

cleanUp 0
